/* eslint-disable */
/**
 * name: @v-charts2/chart
 * version: v1.0.1-alpha.4
 * author: vxhly <pengchengou@gmail.com>
 */
var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
import numerify from "numerify";
import { isFunction, isArray, set, isObject, debounce, isEqual, cloneDeep, camelToKebab, getType, get } from "utils-lite";
import { TitleComponent, TooltipComponent, GridComponent, DatasetComponent, TransformComponent, LegendComponent, DataZoomComponent, VisualMapComponent } from "echarts/components";
import * as echartsLib from "echarts/core";
import { LabelLayout, UniversalTransition } from "echarts/features";
import { SVGRenderer, CanvasRenderer } from "echarts/renderers";
import { computed, h, nextTick, watch, onMounted, onBeforeUnmount } from "vue";
import { LineChart, BarChart, PieChart, CandlestickChart, FunnelChart, GaugeChart, TreeChart, ScatterChart, SankeyChart, RadarChart, MapChart, HeatmapChart } from "echarts/charts";
import "echarts-amap";
import "echarts-liquidfill";
import "echarts-wordcloud";
import "echarts/extension/bmap/bmap";
const name$j = "@v-charts2/chart";
const version$j = "1.0.1-alpha.4";
const name$i = "@v-charts2/core";
const version$i = "1.0.1-alpha.4";
const DEFAULT_THEME = {
  categoryAxis: {
    axisLine: { show: false },
    axisTick: { show: false },
    splitLine: { show: false }
  },
  valueAxis: { axisLine: { show: false } },
  line: { smooth: true },
  legend: { top: 20 },
  grid: {
    containLabel: true,
    top: 100,
    left: 20,
    right: 20,
    bottom: 20
  }
};
const DEFAULT_COLORS = [
  "#19d4ae",
  "#5ab1ef",
  "#fa6e86",
  "#ffb980",
  "#0067a6",
  "#c4b4e4",
  "#d87a80",
  "#9cbbff",
  "#d9d0c7",
  "#87a997",
  "#d49ea2",
  "#5b4947",
  "#7ba3a8"
];
const HEAT_MAP_COLOR = [
  "#313695",
  "#4575b4",
  "#74add1",
  "#abd9e9",
  "#e0f3f8",
  "#ffffbf",
  "#fee090",
  "#fdae61",
  "#f46d43",
  "#d73027",
  "#a50026"
];
const HEAT_BMAP_COLOR = ["blue", "blue", "green", "yellow", "red"];
const itemPoint = (color) => {
  return [
    '<span style="',
    `background-color:${color};`,
    "display: inline-block;",
    "width: 10px;",
    "height: 10px;",
    "border-radius: 50%;",
    "margin-right:2px;",
    '"></span>'
  ].join("");
};
const STATIC_PROPS = [
  "initOptions",
  "loading",
  "dataEmpty",
  "judgeWidth",
  "widthChangeDelay"
];
const ECHARTS_SETTINGS = [
  "grid",
  "dataZoom",
  "visualMap",
  "toolbox",
  "title",
  "legend",
  "xAxis",
  "yAxis",
  "radar",
  "tooltip",
  "axisPointer",
  "brush",
  "geo",
  "timeline",
  "graphic",
  "series",
  "backgroundColor",
  "textStyle"
];
echartsLib.use([
  TitleComponent,
  TooltipComponent,
  GridComponent,
  DatasetComponent,
  TransformComponent,
  LabelLayout,
  UniversalTransition,
  SVGRenderer,
  CanvasRenderer,
  LegendComponent
]);
const getFormated = (val, type, digit, defaultVal = "-") => {
  if (isNaN(val))
    return defaultVal;
  if (!type)
    return val;
  if (isFunction(type))
    return type(val, numerify);
  digit = isNaN(digit) ? 0 : ++digit;
  const digitStr = `.[${new Array(digit).join(0)}]`;
  let formatter = type;
  switch (type) {
    case "KMB":
      formatter = digit ? `0,0${digitStr}a` : "0,0a";
      break;
    case "normal":
      formatter = digit ? `0,0${digitStr}` : "0,0";
      break;
    case "percent":
      formatter = digit ? `0,0${digitStr}%` : "0,0.[00]%";
      break;
  }
  return numerify(val, formatter);
};
const getStackMap = (stack) => {
  const stackMap = {};
  Object.keys(stack).forEach((item) => {
    stack[item].forEach((name2) => {
      stackMap[name2] = item;
    });
  });
  return stackMap;
};
const $get = (url) => {
  return new Promise((resolve, reject) => {
    const xhr = new XMLHttpRequest();
    xhr.open("GET", url);
    xhr.send(null);
    xhr.onload = () => {
      resolve(JSON.parse(xhr.responseText));
    };
    xhr.onerror = () => {
      reject(JSON.parse(xhr.responseText));
    };
  });
};
const mapPromise = {};
const getMapJSON = ({
  position,
  positionJsonLink,
  beforeRegisterMapOnce,
  mapURLProfix
}) => {
  const link = positionJsonLink || `${mapURLProfix}${position}.json`;
  if (!mapPromise[link]) {
    mapPromise[link] = $get(link).then((res) => {
      if (beforeRegisterMapOnce)
        res = beforeRegisterMapOnce(res);
      return res;
    });
  }
  return mapPromise[link];
};
let bmapPromise = null;
let amapPromise = null;
const getBmap = (key, v) => {
  if (!bmapPromise) {
    bmapPromise = new Promise((resolve) => {
      const callbackName = `bmap${Date.now()}`;
      window[callbackName] = resolve;
      const script = document.createElement("script");
      script.src = [
        `https://api.map.baidu.com/api?v=${v || "2.0"}`,
        `ak=${key}`,
        `callback=${callbackName}`
      ].join("&");
      document.body.appendChild(script);
    });
  }
  return bmapPromise;
};
const getAmap = (key, v) => {
  if (!amapPromise) {
    amapPromise = new Promise((resolve) => {
      const callbackName = `amap${Date.now()}`;
      window[callbackName] = resolve;
      const script = document.createElement("script");
      script.src = [
        `https://webapi.amap.com/maps?v=${v || "1.4.3"}`,
        `key=${key}`,
        `callback=${callbackName}`
      ].join("&");
      document.body.appendChild(script);
    });
  }
  return amapPromise;
};
function setArrayValue(arr, index, value) {
  if (arr[index] !== void 0) {
    arr[index].push(value);
  } else {
    arr[index] = [value];
  }
}
function logCopyRight(packageName, packageVersion) {
  console.log(`
 %c author %c vxhly %c %c email %c <pengchengou@gmail.com> %c 

 %c ${name$i} %c v${version$i} %c %c ${packageName} %c v${packageVersion}  
`, "color: #fff; background: #5A5A5A; padding:5px;", "color: #fff; background: #4FC921; padding:5px;", "content:' '", "color: #fff; background: #5A5A5A; padding:5px;", "color: #fff;background: #1890ff; padding:5px 0;", "content:' '", "color: #fff; background: #5A5A5A; padding:5px;", "color: #fff;background: #E77335; padding:5px 0;", "content:' '", "color: #fff; background: #5A5A5A; padding:5px;", "color: #fff;background: #E77335; padding:5px 0;");
}
function getLineXAxis(args) {
  const {
    dimension,
    rows,
    xAxisName,
    axisVisible,
    xAxisType
  } = args;
  return dimension.map((item, index) => ({
    type: xAxisType,
    nameLocation: "middle",
    nameGap: 22,
    name: xAxisName[index] || "",
    axisTick: {
      show: true,
      lineStyle: { color: "#eee" }
    },
    data: rows.map((row) => row[item]),
    show: axisVisible
  }));
}
function getLineSeries(args) {
  const {
    rows,
    axisSite,
    metrics,
    area,
    stack,
    nullAddZero,
    labelMap,
    label,
    itemStyle,
    lineStyle,
    areaStyle,
    dimension
  } = args;
  const series = [];
  const dataTemp = {};
  const stackMap = stack && getStackMap(stack);
  metrics.forEach((item) => {
    dataTemp[item] = [];
  });
  rows.forEach((row) => {
    metrics.forEach((item) => {
      let value = null;
      if (row[item] != null) {
        value = row[item];
      } else if (nullAddZero) {
        value = 0;
      }
      dataTemp[item].push([row[dimension[0]], value]);
    });
  });
  metrics.forEach((item) => {
    const seriesItem = {
      name: labelMap[item] != null ? labelMap[item] : item,
      type: "line",
      data: dataTemp[item]
    };
    if (area) {
      seriesItem.areaStyle = {};
    }
    if (axisSite.right) {
      seriesItem.yAxisIndex = ~axisSite.right.indexOf(item) ? 1 : 0;
    }
    if (stack && stackMap[item])
      seriesItem.stack = stackMap[item];
    if (label)
      seriesItem.label = label;
    if (itemStyle)
      seriesItem.itemStyle = itemStyle;
    if (lineStyle)
      seriesItem.lineStyle = lineStyle;
    if (areaStyle)
      seriesItem.areaStyle = areaStyle;
    series.push(seriesItem);
  });
  return series;
}
function getLineYAxis(args) {
  const {
    yAxisName,
    yAxisType,
    axisVisible,
    scale,
    min,
    max,
    digit
  } = args;
  const yAxisBase = {
    type: "value",
    axisTick: { show: false },
    show: axisVisible
  };
  const yAxis = [];
  for (let i = 0; i < 2; i++) {
    if (yAxisType[i]) {
      yAxis[i] = Object.assign({}, yAxisBase, {
        axisLabel: {
          formatter(val) {
            return getFormated(val, yAxisType[i], digit);
          }
        }
      });
    } else {
      yAxis[i] = Object.assign({}, yAxisBase);
    }
    yAxis[i].name = yAxisName[i] || "";
    yAxis[i].scale = scale[i] || false;
    yAxis[i].min = min[i] || null;
    yAxis[i].max = max[i] || null;
  }
  return yAxis;
}
function getLineTooltip(args) {
  const {
    axisSite,
    yAxisType,
    digit,
    labelMap,
    tooltipFormatter
  } = args;
  const rightItems = axisSite.right || [];
  const rightList = labelMap ? rightItems.map((item) => {
    return labelMap[item] === void 0 ? item : labelMap[item];
  }) : rightItems;
  return {
    trigger: "axis",
    formatter(items) {
      if (tooltipFormatter) {
        return tooltipFormatter.apply(null, arguments);
      }
      const tpl = [];
      const {
        name: name2,
        axisValueLabel
      } = items[0];
      const title = name2 || axisValueLabel;
      tpl.push(`${title}<br>`);
      items.forEach(({
        seriesName,
        data,
        marker
      }) => {
        let showData = null;
        const type = ~rightList.indexOf(seriesName) ? yAxisType[1] : yAxisType[0];
        const itemData = isArray(data) ? data[1] : data;
        showData = getFormated(itemData, type, digit);
        tpl.push(marker);
        tpl.push(`${seriesName}: ${showData}`);
        tpl.push("<br>");
      });
      return tpl.join("");
    }
  };
}
function getLegend$1(args) {
  const {
    metrics,
    legendName,
    labelMap
  } = args;
  if (!legendName && !labelMap) {
    return { data: metrics };
  }
  const data = labelMap ? metrics.map((item) => labelMap[item] == null ? item : labelMap[item]) : metrics;
  return {
    data,
    formatter(name2) {
      return legendName[name2] != null ? legendName[name2] : name2;
    }
  };
}
const line = (columns, rows, settings, extra) => {
  rows = isArray(rows) ? rows : [];
  columns = isArray(columns) ? columns : [];
  const {
    axisSite = {},
    yAxisType = ["normal", "normal"],
    xAxisType = "category",
    yAxisName = [],
    dimension = [columns[0]],
    xAxisName = [],
    axisVisible = true,
    area,
    stack,
    scale = [false, false],
    min = [null, null],
    max = [null, null],
    nullAddZero = false,
    digit = 2,
    legendName = {},
    labelMap = {},
    label,
    itemStyle,
    lineStyle,
    areaStyle
  } = settings;
  const {
    tooltipVisible,
    legendVisible,
    tooltipFormatter
  } = extra;
  let metrics = columns.slice();
  if (axisSite.left && axisSite.right) {
    metrics = axisSite.left.concat(axisSite.right);
  } else if (axisSite.left && !axisSite.right) {
    metrics = axisSite.left;
  } else if (settings.metrics) {
    metrics = settings.metrics;
  } else {
    metrics.splice(columns.indexOf(dimension[0]), 1);
  }
  const legend = legendVisible && getLegend$1({
    metrics,
    legendName,
    labelMap
  });
  const tooltip = tooltipVisible && getLineTooltip({
    axisSite,
    yAxisType,
    digit,
    labelMap,
    xAxisType,
    tooltipFormatter
  });
  const xAxis = getLineXAxis({
    dimension,
    rows,
    xAxisName,
    axisVisible,
    xAxisType
  });
  const yAxis = getLineYAxis({
    yAxisName,
    yAxisType,
    axisVisible,
    scale,
    min,
    max,
    digit
  });
  const series = getLineSeries({
    rows,
    axisSite,
    metrics,
    area,
    stack,
    nullAddZero,
    labelMap,
    label,
    itemStyle,
    lineStyle,
    areaStyle,
    xAxisType,
    dimension
  });
  const options = {
    legend,
    xAxis,
    series,
    yAxis,
    tooltip
  };
  return options;
};
const name$h = "@v-charts2/line";
const version$h = "1.0.1-alpha.4";
const props = {
  data: {
    type: [Object, Array],
    default() {
      return {};
    }
  },
  settings: {
    type: Object,
    default() {
      return {};
    }
  },
  width: {
    type: String,
    default: "auto"
  },
  height: {
    type: String,
    default: "400px"
  },
  beforeConfig: { type: Function },
  afterConfig: { type: Function },
  afterSetOption: { type: Function },
  afterSetOptionOnce: { type: Function },
  events: { type: Object },
  grid: { type: [Object, Array] },
  colors: { type: Array },
  tooltipVisible: {
    type: Boolean,
    default: true
  },
  legendVisible: {
    type: Boolean,
    default: true
  },
  legendPosition: { type: String },
  markLine: { type: Object },
  markArea: { type: Object },
  markPoint: { type: Object },
  visualMap: { type: [Object, Array] },
  dataZoom: { type: [Object, Array] },
  toolbox: { type: [Object, Array] },
  initOptions: {
    type: Object,
    default() {
      return {};
    }
  },
  title: [Object, Array],
  legend: [Object, Array],
  xAxis: [Object, Array],
  yAxis: [Object, Array],
  radar: Object,
  tooltip: Object,
  axisPointer: [Object, Array],
  brush: [Object, Array],
  geo: [Object, Array],
  timeline: [Object, Array],
  graphic: [Object, Array],
  series: [Object, Array],
  backgroundColor: [Object, String],
  textStyle: [Object, Array],
  animation: Object,
  theme: Object,
  themeName: String,
  loading: Boolean,
  dataEmpty: Boolean,
  extend: Object,
  judgeWidth: {
    type: Boolean,
    default: false
  },
  widthChangeDelay: {
    type: Number,
    default: 300
  },
  tooltipFormatter: { type: Function },
  resizeable: {
    type: Boolean,
    default: true
  },
  resizeDelay: {
    type: Number,
    default: 200
  },
  changeDelay: {
    type: Number,
    default: 0
  },
  setOptionOpts: {
    type: [Boolean, Object],
    default: true
  },
  cancelResizeCheck: Boolean,
  notSetUnchange: Array,
  log: Boolean,
  renderer: {
    type: String,
    default: "canvas",
    validator: (val) => {
      return ["svg", "canvas"].indexOf(val) > -1;
    }
  }
};
function setAnimation(options, animation) {
  Object.keys(animation).forEach((key) => {
    options[key] = animation[key];
  });
}
function setExtend(options, extend) {
  Object.keys(extend).forEach((attr) => {
    const value = extend[attr];
    if (~attr.indexOf(".")) {
      set(options, attr, value);
    } else if (typeof value === "function") {
      options[attr] = value(options[attr]);
    } else {
      if (isArray(options[attr]) && isObject(options[attr][0])) {
        options[attr].forEach((option, index) => {
          options[attr][index] = Object.assign({}, option, value);
        });
      } else if (isObject(options[attr])) {
        options[attr] = Object.assign({}, options[attr], value);
      } else {
        options[attr] = value;
      }
    }
  });
}
function setMark(seriesItem, marks) {
  Object.keys(marks).forEach((key) => {
    if (marks[key])
      seriesItem[key] = marks[key];
  });
}
const init = ({
  echarts,
  props: props2,
  canvas,
  options
} = {}) => {
  if (echarts)
    return;
  const {
    registeredEvents,
    _once,
    el
  } = options;
  const themeName = (props2 == null ? void 0 : props2.themeName) || (props2 == null ? void 0 : props2.theme) || DEFAULT_THEME;
  echarts = echartsLib.init(canvas, themeName, __spreadValues({
    renderer: props2 == null ? void 0 : props2.renderer
  }, props2 == null ? void 0 : props2.initOptions));
  if (props2 == null ? void 0 : props2.data) {
    changeHandler({
      props: props2,
      options,
      echarts
    });
  }
  createEventProxy({
    registeredEvents,
    echarts,
    props: props2
  });
  if (props2 == null ? void 0 : props2.resizeable) {
    addResizeListener({
      _once,
      props: props2,
      el,
      echarts
    });
  }
  return { echarts };
};
const changeHandler = ({
  props: props2,
  options,
  echarts
} = {}) => {
  return debounce(dataHandler({
    props: props2,
    options,
    echarts
  }), props2 == null ? void 0 : props2.changeDelay);
};
const resizeHandler = ({
  props: props2,
  el,
  echarts
} = {}) => {
  return debounce(resize({
    props: props2,
    el,
    echarts
  }), props2 == null ? void 0 : props2.resizeDelay);
};
const createEventProxy = ({
  registeredEvents,
  echarts,
  props: props2
}) => {
  const keys = Object.keys((props2 == null ? void 0 : props2.events) || {});
  keys.length && keys.forEach((ev) => {
    if (registeredEvents.indexOf(ev) === -1) {
      registeredEvents.push(ev);
      echarts.on(ev, function(ev2) {
        return function(...args) {
          if (ev2 in props2.events) {
            props2 == null ? void 0 : props2.events[ev2].apply(null, args);
          }
        };
      }(ev));
    }
  });
};
const addResizeListener = ({
  _once,
  props: props2,
  el,
  echarts
}) => {
  window.addEventListener("resize", () => resizeHandler({
    props: props2,
    el,
    echarts
  }));
  _once.onresize = true;
};
const removeResizeListener = ({ _once }) => {
  window.removeEventListener("resize", resizeHandler);
  _once.onresize = false;
};
const dataHandler = ({
  props: props2,
  options,
  echarts
} = {}) => {
  const {
    chartHandler,
    chartColor,
    _once
  } = options;
  if (!chartHandler)
    return;
  let data = props2 == null ? void 0 : props2.data;
  const {
    columns = [],
    rows = []
  } = data;
  const extra = {
    tooltipVisible: props2 == null ? void 0 : props2.tooltipVisible,
    legendVisible: props2 == null ? void 0 : props2.legendVisible,
    echarts,
    color: chartColor,
    tooltipFormatter: props2 == null ? void 0 : props2.tooltipFormatter,
    _once
  };
  if (props2 == null ? void 0 : props2.beforeConfig)
    data = props2 == null ? void 0 : props2.beforeConfig(data);
  const chartOptions = chartHandler(columns, rows, props2 == null ? void 0 : props2.settings, extra);
  if (chartOptions) {
    if (typeof chartOptions.then === "function") {
      chartOptions.then((res) => {
        optionsHandler({
          chartOptions: res,
          props: props2,
          options,
          echarts
        });
      });
    } else {
      optionsHandler({
        chartOptions,
        props: props2,
        options,
        echarts
      });
    }
  }
};
const optionsHandler = ({
  chartOptions,
  props: props2,
  options,
  echarts,
  nextTick: nextTick2
}) => {
  const {
    chartColor,
    _store,
    _once,
    el,
    readyCallback,
    readyOnceCallback
  } = options;
  if ((props2 == null ? void 0 : props2.legendPosition) && chartOptions.legend) {
    chartOptions.legend[props2 == null ? void 0 : props2.legendPosition] = 10;
    if (~["left", "right"].indexOf(props2 == null ? void 0 : props2.legendPosition)) {
      chartOptions.legend.top = "middle";
      chartOptions.legend.orient = "vertical";
    }
  }
  chartOptions.color = chartColor;
  ECHARTS_SETTINGS.forEach((setting) => {
    if (props2 == null ? void 0 : props2[setting])
      chartOptions[setting] = props2 == null ? void 0 : props2[setting];
  });
  if (props2 == null ? void 0 : props2.animation)
    setAnimation(chartOptions, props2 == null ? void 0 : props2.animation);
  if ((props2 == null ? void 0 : props2.markArea) || (props2 == null ? void 0 : props2.markLine) || (props2 == null ? void 0 : props2.markPoint)) {
    const marks = {
      markArea: props2 == null ? void 0 : props2.markArea,
      markLine: props2 == null ? void 0 : props2.markLine,
      markPoint: props2 == null ? void 0 : props2.markPoint
    };
    const series = chartOptions.series;
    if (isArray(series)) {
      series.forEach((item) => {
        setMark(item, marks);
      });
    } else if (isObject(series)) {
      setMark(series, marks);
    }
  }
  if (props2 == null ? void 0 : props2.extend)
    setExtend(chartOptions, props2 == null ? void 0 : props2.extend);
  if (props2 == null ? void 0 : props2.afterConfig)
    chartOptions = props2 == null ? void 0 : props2.afterConfig(chartOptions);
  let setOptionOpts = props2 == null ? void 0 : props2.setOptionOpts;
  if (((props2 == null ? void 0 : props2.settings.bmap) || (props2 == null ? void 0 : props2.settings.amap)) && !isObject(setOptionOpts)) {
    setOptionOpts = false;
  }
  if ((props2 == null ? void 0 : props2.notSetUnchange) && (props2 == null ? void 0 : props2.notSetUnchange.length)) {
    props2 == null ? void 0 : props2.notSetUnchange.forEach((item) => {
      const value = chartOptions[item];
      if (value) {
        if (isEqual(value, _store[item])) {
          chartOptions[item] = void 0;
        } else {
          _store[item] = cloneDeep(value);
        }
      }
    });
    if (isObject(setOptionOpts)) {
      setOptionOpts.notMerge = false;
    } else {
      setOptionOpts = false;
    }
  }
  if (props2 == null ? void 0 : props2.log)
    console.log(chartOptions);
  echarts.setOption(chartOptions, setOptionOpts);
  readyCallback instanceof Function && readyCallback(echarts, chartOptions, echartsLib);
  if (!_once["ready-once"]) {
    _once["ready-once"] = true;
    readyOnceCallback instanceof Function && readyOnceCallback(echarts, chartOptions, echartsLib);
  }
  if (props2 == null ? void 0 : props2.judgeWidth) {
    judgeWidthHandler({
      props: props2,
      el,
      nextTick: nextTick2,
      echarts
    });
  }
  if (props2 == null ? void 0 : props2.afterSetOption) {
    props2 == null ? void 0 : props2.afterSetOption(echarts, chartOptions, echartsLib);
  }
  if ((props2 == null ? void 0 : props2.afterSetOptionOnce) && !_once.afterSetOptionOnce) {
    _once.afterSetOptionOnce = true;
    props2 == null ? void 0 : props2.afterSetOptionOnce(echarts, chartOptions, echartsLib);
  }
};
const judgeWidthHandler = ({
  props: props2,
  el,
  nextTick: nextTick2,
  echarts
}) => {
  if (el.clientWidth || el.clientHeight) {
    resize({
      props: props2,
      el,
      echarts
    });
  } else {
    nextTick2(() => {
      if (el.clientWidth || el.clientHeight) {
        resize({
          props: props2,
          el,
          echarts
        });
      } else {
        setTimeout(() => {
          resize({
            props: props2,
            el,
            echarts
          });
          if (!el.clientWidth || !el.clientHeight) {
            console.warn(" Can't get dom width or height ");
          }
        }, props2 == null ? void 0 : props2.widthChangeDelay);
      }
    });
  }
};
const resize = ({
  props: props2,
  el,
  echarts
} = {}) => {
  if (!(props2 == null ? void 0 : props2.cancelResizeCheck)) {
    if (el && el.clientWidth && el.clientHeight) {
      echartsResize(echarts);
    }
  } else {
    echartsResize(echarts);
  }
};
const echartsResize = (echarts) => {
  echarts && echarts.resize();
};
const themeChange = (initPayload, props2) => {
  let {
    echarts,
    option: { _once }
  } = initPayload;
  clean({
    echarts,
    props: props2,
    _once
  });
  echarts = null;
  init(initPayload);
};
const clean = ({
  echarts,
  props: props2,
  _once
}) => {
  if (props2 == null ? void 0 : props2.resizeable) {
    removeResizeListener({ _once });
  }
  echarts && echarts.dispose();
};
const resizeableHandler = ({
  resizeable,
  _once,
  props: props2,
  el,
  echarts
}) => {
  if (resizeable && !_once.onresize) {
    addResizeListener({
      _once,
      props: props2,
      el,
      echarts
    });
  }
  if (!resizeable && _once.onresize) {
    removeResizeListener({ _once });
  }
};
const createChart$1 = ({
  name: name2,
  chartHandler,
  chartLib
}) => {
  return {
    name: name2,
    props,
    setup(props2, {
      slots,
      emit
    }) {
      let echarts = null;
      const registeredEvents = [];
      const _once = {};
      const _store = {};
      const _watchers = [];
      let stateChangeHandler = null;
      if (!changeHandler) {
        stateChangeHandler = chartLib[props2.settings.type];
      } else {
        stateChangeHandler = changeHandler;
      }
      const canvasStyle = computed(() => {
        return {
          width: props2 == null ? void 0 : props2.width,
          height: props2 == null ? void 0 : props2.height,
          position: "relative"
        };
      });
      const chartColor = computed(() => {
        return (props2 == null ? void 0 : props2.colors) || (props2 == null ? void 0 : props2.theme) && (props2 == null ? void 0 : props2.theme.color) || DEFAULT_COLORS;
      });
      const DataEmpty = h("div", { class: "v-charts-data-empty" }, "\u6682\u65E0\u6570\u636E");
      const Loading = h("div", { class: "v-charts-component-loading" }, [
        h("div", { class: "loader" }, [
          h("div", { class: "loading-spinner" }, [
            h("svg", {
              class: "circular",
              viewBox: "25 25 50 50"
            }, [
              h("circle", {
                class: "path",
                cx: "50",
                cy: "50",
                r: "20",
                fill: "none"
              })
            ])
          ])
        ])
      ]);
      const CANVAS = h("div", {
        style: canvasStyle.value,
        class: { "v-charts-mask-status": (props2 == null ? void 0 : props2.dataEmpty) || (props2 == null ? void 0 : props2.loading) }
      });
      const EL = h("div", {
        class: [camelToKebab(name2)],
        style: canvasStyle.value
      }, [
        CANVAS,
        h(DataEmpty, { style: { display: (props2 == null ? void 0 : props2.dataEmpty) ? "" : "none" } }),
        h(Loading, { style: { display: (props2 == null ? void 0 : props2.loading) ? "" : "none" } }),
        slots.default
      ]);
      const initPayload = {
        echarts,
        props: props2,
        canvas: CANVAS.el,
        options: {
          el: EL.el,
          registeredEvents,
          chartHandler,
          chartColor: chartColor.value,
          _once,
          _store,
          readyCallback: (echarts2, chartOptions, echartsLib2) => {
            emit("ready", echarts2, chartOptions, echartsLib2);
          },
          readyOnceCallback: (echarts2, chartOptions, echartsLib2) => {
            emit("ready-once", echarts2, chartOptions, echartsLib2);
          }
        },
        nextTick
      };
      const nextTickResize = () => {
        nextTick(resize({
          props: props2,
          el: EL.el,
          echarts
        }));
      };
      watch(() => props2 == null ? void 0 : props2.data, (v) => {
        if (v) {
          stateChangeHandler({
            props: props2,
            options: initPayload.options,
            echarts
          });
        }
      }, { deep: true });
      watch(() => props2 == null ? void 0 : props2.settings, (v) => {
        if (v.type && chartLib)
          chartHandler = chartLib[v.type];
        initPayload.options.chartHandler = chartLib[v.type];
        stateChangeHandler({
          props: props2,
          options: initPayload.options,
          echarts
        });
      }, { deep: true });
      watch(() => props2 == null ? void 0 : props2.width, () => nextTickResize());
      watch(() => props2 == null ? void 0 : props2.height, () => nextTickResize());
      watch(() => props2 == null ? void 0 : props2.events, () => createEventProxy({
        registeredEvents,
        echarts,
        props: props2
      }), { deep: true });
      watch(() => props2 == null ? void 0 : props2.theme, () => {
        nextTick(() => {
          initPayload.canvas = CANVAS.el;
          initPayload.options.el = EL.el;
          themeChange(initPayload, props2);
        });
      }, { deep: true });
      watch(() => props2 == null ? void 0 : props2.themeName, () => {
        nextTick(() => {
          initPayload.canvas = CANVAS.el;
          initPayload.options.el = EL.el;
          themeChange(initPayload, props2);
        });
      });
      watch(() => props2 == null ? void 0 : props2.resizeable, (resizeable) => {
        nextTick(() => {
          resizeableHandler({
            resizeable,
            _once,
            props: props2,
            el: EL.el,
            echarts
          });
        });
      });
      const addWatchToProps = ({
        _watchers: _watchers2,
        props: props3,
        options,
        echarts: echarts2
      }) => {
        const watchedVariable = _watchers2.map((watcher) => watcher.expression);
        Object.keys(props3).forEach((prop) => {
          if (!~watchedVariable.indexOf(prop) && !~STATIC_PROPS.indexOf(prop)) {
            const opts = {};
            if (~["[object Object]", "[object Array]"].indexOf(getType(props3[prop]))) {
              opts.deep = true;
            }
            watch(() => prop, () => stateChangeHandler({
              props: props3,
              options,
              echarts: echarts2
            }), opts);
          }
        });
      };
      onMounted(() => {
        initPayload.canvas = CANVAS.el;
        initPayload.options.el = EL.el;
        const { echarts: initEcharts } = init(initPayload);
        echarts = initEcharts;
        addWatchToProps({
          _watchers,
          props: props2,
          options: initPayload.options,
          echarts
        });
      });
      onBeforeUnmount(() => {
        clean({
          echarts,
          props: props2,
          _once
        });
      });
      return () => EL;
    }
  };
};
echartsLib.use([LineChart]);
const VeLine = createChart$1({
  name: "VeLine",
  chartHandler: line
});
VeLine.install = (app) => {
  logCopyRight(name$h, version$h);
  app.component(VeLine.name, VeLine);
};
const VALUE_AXIS_OPACITY = 0.5;
function getBarDimAxis(args) {
  const {
    innerRows,
    dimAxisName,
    dimension,
    axisVisible,
    dimAxisType,
    dims
  } = args;
  return dimension.map((item) => ({
    type: "category",
    name: dimAxisName,
    nameLocation: "middle",
    nameGap: 22,
    data: dimAxisType === "value" ? getValueAxisData(dims) : innerRows.map((row) => row[item]),
    axisLabel: {
      formatter(v) {
        return String(v);
      }
    },
    show: axisVisible
  }));
}
function getValueAxisData(dims) {
  const max = Math.max.apply(null, dims);
  const min = Math.min.apply(null, dims);
  const result = [];
  for (let i = min; i <= max; i++) {
    result.push(i);
  }
  return result;
}
function getBarMeaAxis(args) {
  const {
    meaAxisName,
    meaAxisType,
    axisVisible,
    digit,
    scale,
    min,
    max
  } = args;
  const meaAxisBase = {
    type: "value",
    axisTick: { show: false },
    show: axisVisible
  };
  const meaAxis = [];
  for (let i = 0; i < 2; i++) {
    if (meaAxisType[i]) {
      meaAxis[i] = Object.assign({}, meaAxisBase, {
        axisLabel: {
          formatter(val) {
            return getFormated(val, meaAxisType[i], digit);
          }
        }
      });
    } else {
      meaAxis[i] = Object.assign({}, meaAxisBase);
    }
    meaAxis[i].name = meaAxisName[i] || "";
    meaAxis[i].scale = scale[i] || false;
    meaAxis[i].min = min[i] || null;
    meaAxis[i].max = max[i] || null;
  }
  return meaAxis;
}
function getBarTooltip(args) {
  const {
    axisSite,
    isHistogram,
    meaAxisType,
    digit,
    labelMap
  } = args;
  let secondAxis = isHistogram ? axisSite.right || [] : axisSite.top || [];
  if (labelMap) {
    secondAxis = secondAxis.map((item) => {
      return labelMap[item] === void 0 ? item : labelMap[item];
    });
  }
  return {
    trigger: "axis",
    formatter(items) {
      const tpl = [];
      tpl.push(`${items[0].name}<br>`);
      items.forEach((item) => {
        const seriesName = item.seriesName;
        const type = ~secondAxis.indexOf(seriesName) ? meaAxisType[1] : meaAxisType[0];
        tpl.push(itemPoint(item.color));
        tpl.push(`${seriesName}: `);
        tpl.push(getFormated(item.value, type, digit));
        tpl.push("<br>");
      });
      return tpl.join("");
    }
  };
}
function getValueData(seriesTemp, dims) {
  const max = Math.max.apply(null, dims);
  const min = Math.min.apply(null, dims);
  const result = [];
  for (let i = min; i <= max; i++) {
    const index = dims.indexOf(i);
    if (~index) {
      result.push(seriesTemp[index]);
    } else {
      result.push(null);
    }
  }
  return result;
}
function getBarSeries(args) {
  const {
    innerRows,
    metrics,
    stack,
    axisSite,
    isHistogram,
    labelMap,
    itemStyle,
    label,
    showLine = [],
    dimAxisType,
    barGap,
    opacity,
    dims
  } = args;
  let series = [];
  const seriesTemp = {};
  const secondAxis = isHistogram ? axisSite.right || [] : axisSite.top || [];
  const secondDimAxisIndex = isHistogram ? "yAxisIndex" : "xAxisIndex";
  const stackMap = stack && getStackMap(stack);
  metrics.forEach((item) => {
    seriesTemp[item] = [];
  });
  innerRows.forEach((row) => {
    metrics.forEach((item) => {
      seriesTemp[item].push(row[item]);
    });
  });
  series = Object.keys(seriesTemp).map((item, index) => {
    const data = dimAxisType === "value" ? getValueData(seriesTemp[item], dims) : seriesTemp[item];
    const seriesItem = {
      name: labelMap[item] != null ? labelMap[item] : item,
      type: ~showLine.indexOf(item) ? "line" : "bar",
      data,
      [secondDimAxisIndex]: ~secondAxis.indexOf(item) ? "1" : "0"
    };
    if (stack && stackMap[item])
      seriesItem.stack = stackMap[item];
    if (label)
      seriesItem.label = label;
    if (itemStyle)
      seriesItem.itemStyle = itemStyle;
    let itemOpacity = opacity || get(seriesItem, "itemStyle.opacity");
    if (dimAxisType === "value") {
      seriesItem.barGap = barGap;
      seriesItem.barCategoryGap = "1%";
      if (itemOpacity == null)
        itemOpacity = VALUE_AXIS_OPACITY;
    }
    if (itemOpacity != null) {
      set(seriesItem, "itemStyle.opacity", itemOpacity);
    }
    return seriesItem;
  });
  return series.length ? series : false;
}
function getLegend(args) {
  const {
    metrics,
    labelMap,
    legendName
  } = args;
  if (!legendName && !labelMap)
    return { data: metrics };
  const data = labelMap ? metrics.map((item) => labelMap[item] == null ? item : labelMap[item]) : metrics;
  return {
    data,
    formatter(name2) {
      return legendName[name2] != null ? legendName[name2] : name2;
    }
  };
}
function getDims(rows, dimension) {
  return rows.map((row) => row[dimension[0]]);
}
const bar = (columns, rows, settings, extra) => {
  const innerRows = cloneDeep(rows);
  const {
    axisSite = {},
    dimension = [columns[0]],
    stack = {},
    axisVisible = true,
    digit = 2,
    dataOrder = false,
    scale = [false, false],
    min = [null, null],
    max = [null, null],
    legendName = {},
    labelMap = {},
    label,
    itemStyle,
    showLine,
    barGap = "-100%",
    opacity
  } = settings;
  const {
    tooltipVisible,
    legendVisible
  } = extra;
  let metrics = columns.slice();
  if (axisSite.top && axisSite.bottom) {
    metrics = axisSite.top.concat(axisSite.bottom);
  } else if (axisSite.bottom && !axisSite.right) {
    metrics = axisSite.bottom;
  } else if (settings.metrics) {
    metrics = settings.metrics;
  } else {
    metrics.splice(columns.indexOf(dimension[0]), 1);
  }
  const meaAxisType = settings.xAxisType || ["normal", "normal"];
  const dimAxisType = settings.yAxisType || "category";
  const meaAxisName = settings.xAxisName || [];
  const dimAxisName = settings.yAxisName || "";
  const isHistogram = false;
  if (dataOrder) {
    const {
      label: label2,
      order
    } = dataOrder;
    if (!label2 || !order) {
      console.warn("Need to provide name and order parameters");
    } else {
      innerRows.sort((a, b) => {
        if (order === "desc") {
          return a[label2] - b[label2];
        } else {
          return b[label2] - a[label2];
        }
      });
    }
  }
  const dims = getDims(innerRows, dimension);
  const legend = legendVisible && getLegend({
    metrics,
    labelMap,
    legendName
  });
  const yAxis = getBarDimAxis({
    innerRows,
    dimAxisName,
    dimension,
    axisVisible,
    dimAxisType,
    dims
  });
  const xAxis = getBarMeaAxis({
    meaAxisName,
    meaAxisType,
    axisVisible,
    digit,
    scale,
    min,
    max
  });
  const series = getBarSeries({
    innerRows,
    metrics,
    stack,
    axisSite,
    isHistogram,
    labelMap,
    itemStyle,
    label,
    showLine,
    dimAxisType,
    dimension,
    barGap,
    opacity,
    dims
  });
  const tooltipParams = {
    axisSite,
    isHistogram,
    meaAxisType,
    digit,
    labelMap
  };
  const tooltip = tooltipVisible && getBarTooltip(tooltipParams);
  const options = {
    legend,
    yAxis,
    series,
    xAxis,
    tooltip
  };
  return options;
};
const histogram = (columns, rows, settings, status) => {
  const innerRows = cloneDeep(rows);
  const {
    axisSite = {},
    dimension = [columns[0]],
    stack = {},
    axisVisible = true,
    digit = 2,
    dataOrder = false,
    scale = [false, false],
    min = [null, null],
    max = [null, null],
    labelMap = {},
    legendName = {},
    label,
    itemStyle,
    showLine,
    barGap = "-100%",
    opacity
  } = settings;
  if (dataOrder) {
    const {
      label: label2,
      order
    } = dataOrder;
    if (!label2 || !order) {
      console.warn("Need to provide name and order parameters");
    } else {
      innerRows.sort((a, b) => {
        if (order === "desc") {
          return a[label2] - b[label2];
        } else {
          return b[label2] - a[label2];
        }
      });
    }
  }
  const {
    tooltipVisible,
    legendVisible
  } = status;
  let metrics = columns.slice();
  if (axisSite.left && axisSite.right) {
    metrics = axisSite.left.concat(axisSite.right);
  } else if (axisSite.left && !axisSite.right) {
    metrics = axisSite.left;
  } else if (settings.metrics) {
    metrics = settings.metrics;
  } else {
    metrics.splice(columns.indexOf(dimension[0]), 1);
  }
  const meaAxisType = settings.yAxisType || ["normal", "normal"];
  const dimAxisType = settings.xAxisType || "category";
  const meaAxisName = settings.yAxisName || [];
  const dimAxisName = settings.xAxisName || "";
  const isHistogram = true;
  const dims = getDims(innerRows, dimension);
  const legend = legendVisible && getLegend({
    metrics,
    labelMap,
    legendName
  });
  const xAxis = getBarDimAxis({
    innerRows,
    dimAxisName,
    dimension,
    axisVisible,
    dimAxisType,
    dims
  });
  const yAxis = getBarMeaAxis({
    meaAxisName,
    meaAxisType,
    axisVisible,
    digit,
    scale,
    min,
    max
  });
  const series = getBarSeries({
    innerRows,
    metrics,
    stack,
    axisSite,
    isHistogram,
    labelMap,
    itemStyle,
    label,
    showLine,
    dimAxisType,
    dimension,
    barGap,
    opacity,
    dims
  });
  const tooltipParams = {
    axisSite,
    isHistogram,
    meaAxisType,
    digit,
    labelMap
  };
  const tooltip = tooltipVisible && getBarTooltip(tooltipParams);
  const options = {
    legend,
    yAxis,
    series,
    xAxis,
    tooltip
  };
  return options;
};
const name$g = "@v-charts2/bar";
const version$g = "1.0.1-alpha.4";
echartsLib.use([BarChart]);
const VeBar = createChart$1({
  name: "VeBar",
  chartHandler: bar
});
VeBar.install = (app) => {
  logCopyRight(name$g, version$g);
  app.component(VeBar.name, VeBar);
};
const name$f = "@v-charts2/histogram";
const version$f = "1.0.1-alpha.4";
echartsLib.use([BarChart]);
const VeHistogram = createChart$1({
  name: "VeHistogram",
  chartHandler: histogram
});
VeHistogram.install = (app) => {
  logCopyRight(name$f, version$f);
  app.component(VeHistogram.name, VeHistogram);
};
const pieRadius = 100;
const ringRadius = [80, 100];
const roseRingRadius = [20, 100];
const pieOffsetY = 200;
function getPieSeries(args) {
  const {
    innerRows,
    dataType,
    percentShow,
    dimension,
    metrics,
    radius,
    offsetY,
    selectedMode,
    hoverAnimation,
    digit,
    roseType,
    label,
    level,
    limitShowNum,
    isRing,
    labelLine,
    itemStyle
  } = args;
  const series = [];
  const levelTemp = {};
  const rowsTemp = [];
  if (level) {
    level.forEach((levelItems, index) => {
      levelItems.forEach((item) => {
        setArrayValue(levelTemp, item, index);
      });
    });
    innerRows.forEach((row) => {
      const itemLevel = levelTemp[row[dimension]];
      if (itemLevel && itemLevel.length) {
        itemLevel.forEach((levelItem) => {
          setArrayValue(rowsTemp, levelItem, row);
        });
      }
    });
  } else {
    rowsTemp.push(innerRows);
  }
  const seriesBase = {
    type: "pie",
    selectedMode,
    emphasis: { scale: hoverAnimation },
    roseType,
    center: ["50%", offsetY]
  };
  const rowsTempLength = rowsTemp.length;
  rowsTemp.forEach((dataRows, index) => {
    const seriesItem = Object.assign({ data: [] }, seriesBase);
    const centerWidth = radius / rowsTempLength;
    if (!index) {
      seriesItem.radius = isRing ? radius : centerWidth;
    } else {
      const outerWidth = centerWidth + radius / (2 * rowsTempLength) * (2 * index - 1);
      const innerWidth = outerWidth + radius / (2 * rowsTempLength);
      seriesItem.radius = [outerWidth, innerWidth];
    }
    if (rowsTempLength > 1 && index === 0) {
      seriesItem.label = { position: "inner" };
    }
    if (label)
      seriesItem.label = label;
    if (labelLine)
      seriesItem.labelLine = labelLine;
    if (itemStyle)
      seriesItem.itemStyle = itemStyle;
    if (percentShow) {
      seriesItem.label = {
        show: true,
        position: rowsTempLength > 1 && index === 0 ? "inner" : "outside",
        formatter(item) {
          const tpl = [];
          tpl.push(`${item.name}:`);
          tpl.push(getFormated(item.value, dataType, digit));
          tpl.push(`(${item.percent}%)`);
          return tpl.join(" ");
        }
      };
    }
    seriesItem.data = dataRows.map((row) => ({
      name: row[dimension],
      value: row[metrics]
    }));
    series.push(seriesItem);
  });
  if (limitShowNum && limitShowNum < series[0].data.length) {
    const firstData = series[0].data;
    const remainArr = firstData.slice(limitShowNum, firstData.length);
    let sum = 0;
    remainArr.forEach((item) => {
      sum += item.value;
    });
    series[0].data = firstData.slice(0, limitShowNum);
    series[0].data.push({
      name: "\u5176\u4ED6",
      value: sum
    });
  }
  return series;
}
function getPieLegend(args) {
  const {
    innerRows,
    dimension,
    legendLimit,
    legendName,
    level,
    limitShowNum
  } = args;
  let legend = [];
  const levelTemp = [];
  if (level) {
    level.forEach((levelItem) => {
      levelItem.forEach((item) => {
        levelTemp.push(item);
      });
    });
    legend = levelTemp;
  } else if (limitShowNum && limitShowNum < innerRows.length) {
    for (let i = 0; i < limitShowNum; i++) {
      legend.push(innerRows[i][dimension]);
    }
    legend.push("\u5176\u4ED6");
  } else {
    legend = innerRows.map((row) => row[dimension]);
  }
  if (legend.length) {
    return {
      data: legend,
      show: legend.length < legendLimit,
      formatter(name2) {
        return legendName[name2] != null ? legendName[name2] : name2;
      }
    };
  } else {
    return false;
  }
}
function getPieTooltip(args) {
  const {
    dataType,
    innerRows,
    limitShowNum,
    digit,
    metrics,
    dimension
  } = args;
  let sum = 0;
  const remainArr = innerRows.map((row) => {
    sum += row[metrics];
    return {
      name: row[dimension],
      value: row[metrics]
    };
  }).slice(limitShowNum, innerRows.length);
  return {
    formatter(item) {
      const tpl = [];
      tpl.push(itemPoint(item.color));
      if (limitShowNum && item.name === "\u5176\u4ED6") {
        tpl.push("\u5176\u4ED6:");
        remainArr.forEach(({
          name: name2,
          value
        }) => {
          const percent = getFormated(value / sum, "percent");
          tpl.push(`<br>${name2}:`);
          tpl.push(getFormated(value, dataType, digit));
          tpl.push(`(${percent})`);
        });
      } else {
        tpl.push(`${item.name}:`);
        tpl.push(getFormated(item.value, dataType, digit));
        tpl.push(`(${item.percent}%)`);
      }
      return tpl.join(" ");
    }
  };
}
const pie = (columns, rows, settings, extra, isRing) => {
  const innerRows = cloneDeep(rows);
  const {
    dataType = "normal",
    percentShow,
    dimension = columns[0],
    metrics = columns[1],
    roseType = false,
    radius = isRing ? roseType ? roseRingRadius : ringRadius : pieRadius,
    offsetY = pieOffsetY,
    legendLimit = 30,
    selectedMode = false,
    hoverAnimation = true,
    digit = 2,
    legendName = {},
    label = false,
    level = false,
    limitShowNum = 0,
    labelLine,
    itemStyle
  } = settings;
  const {
    tooltipVisible,
    legendVisible
  } = extra;
  if (limitShowNum)
    innerRows.sort((a, b) => b[metrics] - a[metrics]);
  const seriesParams = {
    innerRows,
    dataType,
    percentShow,
    dimension,
    metrics,
    radius,
    offsetY,
    selectedMode,
    hoverAnimation,
    digit,
    roseType,
    label,
    level,
    legendName,
    limitShowNum,
    isRing,
    labelLine,
    itemStyle
  };
  const series = getPieSeries(seriesParams);
  const legendParams = {
    innerRows,
    dimension,
    legendLimit,
    legendName,
    level,
    limitShowNum
  };
  const legend = legendVisible && getPieLegend(legendParams);
  const tooltip = tooltipVisible && getPieTooltip({
    dataType,
    innerRows,
    limitShowNum,
    digit,
    metrics,
    dimension
  });
  const options = {
    series,
    legend,
    tooltip
  };
  return options;
};
const ring = (columns, rows, settings, extra) => {
  return pie(columns, rows, settings, extra, true);
};
const name$e = "@v-charts2/pie";
const version$e = "1.0.1-alpha.4";
echartsLib.use([PieChart]);
const VePie = createChart$1({
  name: "VePie",
  chartHandler: pie
});
VePie.install = (app) => {
  logCopyRight(name$e, version$e);
  app.component(VePie.name, VePie);
};
const name$d = "@v-charts2/ring";
const version$d = "1.0.1-alpha.4";
echartsLib.use([PieChart]);
const VeRing = createChart$1({
  name: "VeRing",
  chartHandler: ring
});
VeRing.install = (app) => {
  logCopyRight(name$d, version$d);
  app.component(VeRing.name, VeRing);
};
const DEFAULT_MA = [5, 10, 20, 30];
const DEFAULT_K_NAME = "\u65E5K";
const DEFAULT_DOWN_COLOR = "#ec0000";
const DEFAULT_UP_COLOR = "#00da3c";
const DEFAULT_START = 50;
const DEFAULT_END = 100;
const SHOW_FALSE = { show: false };
function getCandleLegend(args) {
  const {
    showMA,
    MA,
    legendName,
    labelMap
  } = args;
  let data = [DEFAULT_K_NAME];
  if (showMA)
    data = data.concat(MA.map((v) => `MA${v}`));
  if (labelMap)
    data = data.map((v) => labelMap[v] == null ? v : labelMap[v]);
  return {
    data,
    formatter(name2) {
      return legendName[name2] != null ? legendName[name2] : name2;
    }
  };
}
function getCandleTooltip(args) {
  const {
    metrics,
    dataType,
    digit,
    labelMap
  } = args;
  return {
    trigger: "axis",
    axisPointer: { type: "cross" },
    position(pos, params, el, elRect, size) {
      const result = { top: 10 };
      const side = pos[0] < size.viewSize[0] / 2 ? "right" : "left";
      result[side] = 60;
      return result;
    },
    formatter(options) {
      const tpl = [];
      tpl.push(`${options[0].axisValue}<br>`);
      options.forEach((option) => {
        const {
          data,
          seriesName,
          componentSubType,
          color
        } = option;
        const name2 = labelMap[seriesName] == null ? seriesName : labelMap[seriesName];
        tpl.push(`${itemPoint(color)} ${name2}: `);
        if (componentSubType === "candlestick") {
          tpl.push("<br>");
          metrics.slice(0, 4).forEach((m, i) => {
            const name3 = labelMap[m] != null ? labelMap[m] : m;
            const val = getFormated(data[i + 1], dataType, digit);
            tpl.push(`- ${name3}: ${val}<br>`);
          });
        } else if (componentSubType === "line") {
          const val = getFormated(data, dataType, digit);
          tpl.push(`${val}<br>`);
        } else if (componentSubType === "bar") {
          const val = getFormated(data[1], dataType, digit);
          tpl.push(`${val}<br>`);
        }
      });
      return tpl.join("");
    }
  };
}
function getCandleVisualMap(args) {
  const {
    downColor,
    upColor,
    MA,
    showMA
  } = args;
  return {
    show: false,
    seriesIndex: showMA ? 1 + MA.length : 1,
    dimension: 2,
    pieces: [
      {
        value: 1,
        color: downColor
      },
      {
        value: -1,
        color: upColor
      }
    ]
  };
}
function getCandleGrid(args) {
  const { showVol } = args;
  return [
    {
      left: "10%",
      right: "8%",
      top: "15%",
      height: showVol ? "55%" : "75%",
      containLabel: false
    },
    {
      left: "10%",
      right: "8%",
      top: "75%",
      height: "20%",
      containLabel: false
    }
  ];
}
function getCandleXAxis(args) {
  const { dims: data } = args;
  const type = "category";
  const scale = true;
  const boundaryGap = false;
  const splitLine = SHOW_FALSE;
  const axisLine = { onZero: false };
  const axisTick = SHOW_FALSE;
  const axisLabel = SHOW_FALSE;
  const min = "dataMin";
  const max = "dataMax";
  const gridIndex = 1;
  return [
    {
      type,
      data,
      scale,
      boundaryGap,
      axisLine,
      splitLine,
      min,
      max
    },
    {
      type,
      gridIndex,
      data,
      scale,
      boundaryGap,
      axisLine,
      axisTick,
      splitLine,
      axisLabel,
      min,
      max
    }
  ];
}
function getCandleYAxis(args) {
  const {
    dataType,
    digit
  } = args;
  const scale = true;
  const gridIndex = 1;
  const splitNumber = 2;
  const axisLine = SHOW_FALSE;
  const axisTick = SHOW_FALSE;
  const axisLabel = SHOW_FALSE;
  const splitLine = SHOW_FALSE;
  const formatter = (val) => {
    return getFormated(val, dataType, digit);
  };
  return [
    {
      scale,
      axisTick,
      axisLabel: { formatter }
    },
    {
      scale,
      gridIndex,
      splitNumber,
      axisLine,
      axisTick,
      splitLine,
      axisLabel
    }
  ];
}
function getCandleDataZoom(args) {
  const {
    start,
    end
  } = args;
  return [
    {
      type: "inside",
      xAxisIndex: [0, 1],
      start,
      end
    },
    {
      show: true,
      xAxisIndex: [0, 1],
      type: "slider",
      top: "85%",
      start,
      end
    }
  ];
}
function getCandleSeries(args) {
  const {
    values,
    volumes,
    upColor,
    downColor,
    showMA,
    MA,
    showVol,
    labelMap,
    digit,
    itemStyle
  } = args;
  const style = itemStyle || {
    color: upColor,
    color0: downColor,
    borderColor: null,
    borderColor0: null
  };
  const lineStyle = { opacity: 0.5 };
  const series = [
    {
      name: labelMap[DEFAULT_K_NAME] == null ? DEFAULT_K_NAME : labelMap[DEFAULT_K_NAME],
      type: "candlestick",
      data: values,
      itemStyle: style
    }
  ];
  if (showMA) {
    MA.forEach((d) => {
      const name2 = `MA${d}`;
      series.push({
        name: labelMap[name2] == null ? name2 : labelMap[name2],
        data: calculateMA(d, values, digit),
        type: "line",
        lineStyle,
        smooth: true
      });
    });
  }
  if (showVol) {
    series.push({
      name: "Volume",
      type: "bar",
      xAxisIndex: 1,
      yAxisIndex: 1,
      data: volumes
    });
  }
  return series;
}
function calculateMA(dayCount, data, digit) {
  const result = [];
  data.forEach((d, i) => {
    if (i < dayCount) {
      result.push("-");
    } else {
      let sum = 0;
      for (let j = 0; j < dayCount; j++)
        sum += data[i - j][1];
      result.push(+(sum / dayCount).toFixed(digit));
    }
  });
  return result;
}
const candle = (columns, rows, settings, status) => {
  const {
    dimension = columns[0],
    metrics = columns.slice(1, 6),
    digit = 2,
    itemStyle,
    labelMap = {},
    legendName = {},
    MA = DEFAULT_MA,
    showMA = false,
    showVol = false,
    showDataZoom = false,
    downColor = DEFAULT_DOWN_COLOR,
    upColor = DEFAULT_UP_COLOR,
    start = DEFAULT_START,
    end = DEFAULT_END,
    dataType
  } = settings;
  const {
    tooltipVisible,
    legendVisible
  } = status;
  const isLiteData = isArray(rows[0]);
  const dims = [];
  const values = [];
  const volumes = [];
  const candleMetrics = metrics.slice(0, 4);
  const volumeMetrics = metrics[4];
  if (isLiteData) {
    rows.forEach((row) => {
      const itemResult = [];
      dims.push(row[columns.indexOf(dimension)]);
      candleMetrics.forEach((item) => {
        itemResult.push(row[columns.indexOf(item)]);
      });
      values.push(itemResult);
      if (volumeMetrics)
        volumes.push(row[columns.indexOf(volumeMetrics)]);
    });
  } else {
    rows.forEach((row, index) => {
      const itemResult = [];
      dims.push(row[dimension]);
      candleMetrics.forEach((item) => {
        itemResult.push(row[item]);
      });
      values.push(itemResult);
      if (volumeMetrics) {
        const status2 = row[metrics[0]] > row[metrics[1]] ? 1 : -1;
        volumes.push([index, row[volumeMetrics], status2]);
      }
    });
  }
  const legend = legendVisible && getCandleLegend({
    showMA,
    MA,
    legendName,
    labelMap
  });
  const tooltip = tooltipVisible && getCandleTooltip({
    metrics,
    dataType,
    digit,
    labelMap
  });
  const visualMap = showVol && getCandleVisualMap({
    downColor,
    upColor,
    MA,
    showMA
  });
  const dataZoom = showDataZoom && getCandleDataZoom({
    start,
    end
  });
  const grid = getCandleGrid({ showVol });
  const xAxis = getCandleXAxis({ dims });
  const yAxis = getCandleYAxis({
    dataType,
    digit
  });
  const series = getCandleSeries({
    values,
    volumes,
    upColor,
    downColor,
    showMA,
    MA,
    showVol,
    labelMap,
    digit,
    itemStyle
  });
  const axisPointer = { link: { xAxisIndex: "all" } };
  return {
    legend,
    tooltip,
    visualMap,
    grid,
    xAxis,
    yAxis,
    dataZoom,
    series,
    axisPointer
  };
};
const name$c = "@v-charts2/candle";
const version$c = "1.0.1-alpha.4";
echartsLib.use([LineChart, BarChart, CandlestickChart, DataZoomComponent, VisualMapComponent]);
const VeCandle = createChart$1({
  name: "VeCandle",
  chartHandler: candle
});
VeCandle.install = (app) => {
  logCopyRight(name$c, version$c);
  app.component(VeCandle.name, VeCandle);
};
function getFunnelTooltip(dataType, digit) {
  return {
    trigger: "item",
    formatter(item) {
      const tpl = [];
      tpl.push(itemPoint(item.color));
      tpl.push(`${item.name}: ${getFormated(item.data.realValue, dataType, digit)}`);
      return tpl.join("");
    }
  };
}
function getFunnelLegend(args) {
  const {
    data,
    legendName
  } = args;
  return {
    data,
    formatter(name2) {
      return legendName[name2] != null ? legendName[name2] : name2;
    }
  };
}
function getFunnelSeries(args) {
  const {
    dimension,
    metrics,
    rows,
    sequence,
    ascending,
    label,
    labelLine,
    itemStyle,
    filterZero,
    useDefaultOrder
  } = args;
  const series = { type: "funnel" };
  let innerRows = rows.sort((a, b) => {
    return sequence.indexOf(a[dimension]) - sequence.indexOf(b[dimension]);
  });
  if (filterZero) {
    innerRows = innerRows.filter((row) => {
      return row[metrics];
    });
  }
  let falseFunnel = false;
  innerRows.some((row, index) => {
    if (index && row[metrics] > innerRows[index - 1][metrics]) {
      falseFunnel = true;
      return true;
    }
    return false;
  });
  const step = 100 / innerRows.length;
  if (falseFunnel && !useDefaultOrder) {
    series.data = innerRows.slice().reverse().map((row, index) => ({
      name: row[dimension],
      value: (index + 1) * step,
      realValue: row[metrics]
    }));
  } else {
    series.data = innerRows.map((row) => ({
      name: row[dimension],
      value: row[metrics],
      realValue: row[metrics]
    }));
  }
  if (ascending)
    series.sort = "ascending";
  if (label)
    series.label = label;
  if (labelLine)
    series.labelLine = labelLine;
  if (itemStyle)
    series.itemStyle = itemStyle;
  return series;
}
const funnel = (outerColumns, outerRows, settings, extra) => {
  const columns = outerColumns.slice();
  const rows = outerRows.slice();
  const {
    dataType = "normal",
    dimension = columns[0],
    sequence = rows.map((row) => row[dimension]),
    digit = 2,
    ascending,
    label,
    labelLine,
    legendName = {},
    itemStyle,
    filterZero,
    useDefaultOrder
  } = settings;
  const {
    tooltipVisible,
    legendVisible
  } = extra;
  let metrics;
  if (settings.metrics) {
    metrics = settings.metrics;
  } else {
    const metricsTemp = columns.slice();
    metricsTemp.splice(columns.indexOf(dimension), 1);
    metrics = metricsTemp[0];
  }
  const tooltip = tooltipVisible && getFunnelTooltip(dataType, digit);
  const legend = legendVisible && getFunnelLegend({
    data: sequence,
    legendName
  });
  const series = getFunnelSeries({
    dimension,
    metrics,
    rows,
    sequence,
    ascending,
    label,
    labelLine,
    itemStyle,
    filterZero,
    useDefaultOrder
  });
  const options = {
    tooltip,
    legend,
    series
  };
  return options;
};
const name$b = "@v-charts2/funnel";
const version$b = "1.0.1-alpha.4";
echartsLib.use([FunnelChart]);
const VeFunnel = createChart$1({
  name: "VeFunnel",
  chartHandler: funnel
});
VeFunnel.install = (app) => {
  logCopyRight(name$b, version$b);
  app.component(VeFunnel.name, VeFunnel);
};
const amap = (_, __, settings, extra) => {
  const {
    key,
    v,
    amap: amap2,
    useOuterMap
  } = settings;
  const { _once } = extra;
  const registerSign = "amap_register";
  if (!key && !useOuterMap)
    console.warn("settings.key must be a string.");
  if (_once[registerSign])
    return {};
  _once[registerSign] = true;
  if (useOuterMap)
    return { amap: amap2 };
  return getAmap(key, v).then((_2) => {
    return { amap: amap2 };
  });
};
const name$a = "@v-charts2/amap";
const version$a = "1.0.1-alpha.4";
const VeAmap = createChart$1({
  name: "VeAmap",
  chartHandler: amap
});
VeAmap.install = (app) => {
  logCopyRight(name$a, version$a);
  app.component(VeAmap.name, VeAmap);
};
function getTooltip$5(args) {
  const {
    tooltipFormatter,
    dataType,
    digit
  } = args;
  return {
    formatter(options) {
      const {
        seriesName,
        data: {
          value,
          name: name2
        }
      } = options;
      if (tooltipFormatter) {
        return tooltipFormatter.apply(null, arguments);
      }
      const tpl = [];
      tpl.push(`${seriesName}: `);
      tpl.push(`${getFormated(value, dataType[seriesName], digit)} ${name2}`);
      return tpl.join("");
    }
  };
}
function getSeries$5(args) {
  const {
    rows,
    dimension,
    metrics,
    digit,
    dataType,
    labelMap,
    seriesMap,
    dataName
  } = args;
  const series = rows.map((row) => {
    const label = row[dimension];
    const seriesItem = seriesMap[label];
    const result = {
      type: "gauge",
      name: labelMap[label] != null ? labelMap[label] : label,
      data: [
        {
          name: dataName[label] || "",
          value: row[metrics]
        }
      ],
      detail: {
        formatter(v) {
          return getFormated(v, dataType[label], digit);
        }
      },
      axisLabel: {
        formatter(v) {
          return getFormated(v, dataType[label], digit);
        }
      }
    };
    if (seriesItem) {
      Object.keys(seriesItem).forEach((key) => {
        if (isObject(result[key])) {
          Object.assign(result[key], seriesItem[key]);
        } else {
          result[key] = seriesItem[key];
        }
      });
    }
    return result;
  });
  return series;
}
const gauge = (columns, rows, settings, extra) => {
  const {
    dimension = columns[0],
    metrics = columns[1],
    digit = 2,
    dataType = {},
    labelMap = {},
    seriesMap = {},
    dataName = {}
  } = settings;
  const {
    tooltipFormatter,
    tooltipVisible
  } = extra;
  const tooltip = tooltipVisible && getTooltip$5({
    tooltipFormatter,
    dataType
  });
  const series = getSeries$5({
    rows,
    dimension,
    metrics,
    digit,
    dataType,
    labelMap,
    seriesMap,
    dataName
  });
  return {
    tooltip,
    series
  };
};
const name$9 = "@v-charts2/gauge";
const version$9 = "1.0.1-alpha.4";
echartsLib.use([GaugeChart]);
const VeGauge = createChart$1({
  name: "VeGauge",
  chartHandler: gauge
});
VeGauge.install = (app) => {
  logCopyRight(name$9, version$9);
  app.component(VeGauge.name, VeGauge);
};
function getTooltip$4(args) {
  const {
    tooltipFormatter,
    dataType,
    digit
  } = args;
  return {
    show: true,
    formatter(options) {
      const {
        seriesName,
        value
      } = options;
      if (tooltipFormatter) {
        return tooltipFormatter.apply(null, arguments);
      }
      return [
        `${seriesName}: `,
        getFormated(value, dataType, digit)
      ].join("");
    }
  };
}
function getSeries$4(args) {
  const {
    dimension,
    metrics,
    seriesMap,
    rows,
    wave
  } = args;
  let itemWave = wave;
  const len = isArray(seriesMap) ? seriesMap.length : 0;
  return rows.slice().map((item, index) => {
    let data = [];
    let result = { type: "liquidFill" };
    const name2 = item[dimension];
    const val = Number(item[metrics]);
    let itemMap = {};
    if (isArray(seriesMap)) {
      itemMap = !seriesMap[index] ? seriesMap[len - 1] : seriesMap[index];
    } else if (isObject(seriesMap[name2])) {
      itemMap = seriesMap[name2];
    }
    if (isArray(wave) && isArray(wave[0])) {
      itemWave = isArray(wave[index]) ? wave[index] : wave[wave.length - 1];
    }
    data.push({ value: val });
    if (itemWave && itemWave.length) {
      data = data.concat(itemWave.map((val2) => ({ value: val2 })));
    }
    result = Object.assign(result, {
      data,
      name: name2
    }, itemMap);
    return result;
  });
}
const liquidfill = (columns, rows, settings, extra) => {
  const {
    dimension = columns[0],
    metrics = columns[1],
    seriesMap = {},
    dataType = "percent",
    digit = 2,
    wave = []
  } = settings;
  const {
    tooltipVisible,
    tooltipFormatter
  } = extra;
  const tooltip = tooltipVisible && getTooltip$4({
    tooltipFormatter,
    dataType,
    digit
  });
  const series = getSeries$4({
    rows,
    columns,
    dimension,
    metrics,
    seriesMap,
    wave
  });
  return {
    tooltip,
    series
  };
};
const name$8 = "@v-charts2/liquidfill";
const version$8 = "1.0.1-alpha.4";
const VeLiquidfill = createChart$1({
  name: "VeLiquidfill",
  chartHandler: liquidfill
});
VeLiquidfill.install = (app) => {
  logCopyRight(name$8, version$8);
  app.component(VeLiquidfill.name, VeLiquidfill);
};
function getSeries$3(args) {
  const {
    dimension,
    metrics,
    rows,
    color,
    sizeMax,
    sizeMin,
    shape
  } = args;
  const baseType = {
    type: "wordCloud",
    textStyle: {
      color: !isArray(color) && !!color ? color : () => {
        return "rgb(" + [
          Math.round(Math.random() * 160),
          Math.round(Math.random() * 160),
          Math.round(Math.random() * 160)
        ].join(",") + ")";
      }
    },
    shape,
    sizeRange: [sizeMin, sizeMax]
  };
  const len = isArray(color) ? color.length : 0;
  const data = rows.slice().map((row) => {
    const text = {
      name: row[dimension],
      value: row[metrics]
    };
    if (len > 0) {
      text.textStyle = { color: color[Math.floor(Math.random() * len)] };
    }
    return text;
  });
  baseType.data = data;
  return [baseType];
}
function getTooltip$3(args) {
  const { tooltipFormatter } = args;
  return {
    show: true,
    formatter(params) {
      const {
        data: {
          name: name2,
          value
        }
      } = params;
      if (tooltipFormatter) {
        return tooltipFormatter.apply(null, params);
      }
      return `${name2}: ${value}`;
    }
  };
}
const wordcloud = (columns, rows, settings, extra) => {
  const {
    dimension = columns[0],
    metrics = columns[1],
    color = "",
    sizeMax = 60,
    sizeMin = 12,
    shape = "circle"
  } = settings;
  const {
    tooltipVisible,
    tooltipFormatter
  } = extra;
  const series = getSeries$3({
    dimension,
    metrics,
    rows,
    color,
    sizeMax,
    sizeMin,
    shape
  });
  const tooltip = tooltipVisible && getTooltip$3({ tooltipFormatter });
  return {
    series,
    tooltip
  };
};
const name$7 = "@v-charts2/wordcloud";
const version$7 = "1.0.1-alpha.4";
const VeWordcloud = createChart$1({
  name: "VeWordcloud",
  chartHandler: wordcloud
});
VeWordcloud.install = (app) => {
  logCopyRight(name$7, version$7);
  app.component(VeWordcloud.name, VeWordcloud);
};
function getWaterfallTooltip(dataType, digit) {
  return {
    trigger: "axis",
    axisPointer: { type: "shadow" },
    formatter(items) {
      const item = items[1];
      return [
        `${item.name}<br/>${item.seriesName} :`,
        `${getFormated(item.value, dataType, digit)}`
      ].join("");
    }
  };
}
function getWaterfallXAxis(args) {
  const {
    dimension,
    rows,
    remainStatus,
    totalName,
    remainName,
    labelMap,
    xAxisName,
    axisVisible
  } = args;
  let xAxisData = [totalName].concat(rows.map((row) => row[dimension]));
  if (remainStatus === "have-remain") {
    xAxisData = xAxisData.concat([remainName]);
  }
  return {
    type: "category",
    name: labelMap && labelMap[xAxisName] || xAxisName,
    splitLine: { show: false },
    data: xAxisData,
    show: axisVisible
  };
}
function getWaterfallYAxis(args) {
  const {
    dataType,
    yAxisName,
    axisVisible,
    digit,
    labelMap
  } = args;
  return {
    type: "value",
    name: labelMap[yAxisName] != null ? labelMap[yAxisName] : yAxisName,
    axisTick: { show: false },
    axisLabel: {
      formatter(val) {
        return getFormated(val, dataType, digit);
      }
    },
    show: axisVisible
  };
}
function getWaterfallSeries(args) {
  const {
    dataType,
    rows,
    metrics,
    totalNum,
    remainStatus,
    dataSum,
    digit
  } = args;
  const seriesBase = {
    type: "bar",
    stack: "\u603B\u91CF"
  };
  let dataSumTemp = dataSum;
  let totalNumTemp = totalNum;
  let assistData;
  let mainData;
  const rowData = rows.map((row) => row[metrics]);
  if (remainStatus === "have-remain") {
    assistData = [0].concat(rows.map((row) => {
      totalNumTemp -= row[metrics];
      return totalNumTemp;
    })).concat([0]);
    mainData = [totalNum].concat(rowData).concat([totalNum - dataSum]);
  } else {
    assistData = [0].concat(rows.map((row) => {
      dataSumTemp -= row[metrics];
      return dataSumTemp;
    }));
    mainData = [dataSum].concat(rowData);
  }
  const series = [];
  series.push(Object.assign({
    name: "\u8F85\u52A9",
    itemStyle: { opacity: 0 },
    emphasis: { itemStyle: { opacity: 0 } },
    data: assistData
  }, seriesBase));
  series.push(Object.assign({
    name: "\u6570\u503C",
    label: {
      show: true,
      position: "top",
      formatter(item) {
        return getFormated(item.value, dataType, digit);
      }
    },
    data: mainData
  }, seriesBase));
  return series;
}
function getWaterfallRemainStatus(dataSum, totalNum) {
  if (!totalNum)
    return "not-total";
  return totalNum > dataSum ? "have-remain" : "none-remain";
}
const waterfall = (columns, rows, settings, extra) => {
  const {
    dataType = "normal",
    dimension = columns[0],
    totalName = "\u603B\u8BA1",
    totalNum,
    remainName = "\u5176\u4ED6",
    xAxisName = dimension,
    labelMap = {},
    axisVisible = true,
    digit = 2
  } = settings;
  const { tooltipVisible } = extra;
  const metricsTemp = columns.slice();
  metricsTemp.splice(metricsTemp.indexOf(dimension), 1);
  const metrics = metricsTemp[0];
  const yAxisName = metrics;
  const tooltip = tooltipVisible && getWaterfallTooltip(dataType, digit);
  const dataSum = parseFloat(rows.reduce((pre, cur) => {
    return pre + Number(cur[metrics]);
  }, 0).toFixed(digit));
  const remainStatus = getWaterfallRemainStatus(dataSum, totalNum);
  const xAxisParams = {
    dimension,
    rows,
    remainStatus,
    totalName,
    remainName,
    xAxisName,
    labelMap,
    axisVisible
  };
  const xAxis = getWaterfallXAxis(xAxisParams);
  const yAxis = getWaterfallYAxis({
    dataType,
    yAxisName,
    axisVisible,
    digit,
    labelMap
  });
  const seriesParams = {
    dataType,
    rows,
    dimension,
    metrics,
    totalNum,
    remainStatus,
    dataSum,
    digit
  };
  const series = getWaterfallSeries(seriesParams);
  const options = {
    tooltip,
    xAxis,
    yAxis,
    series
  };
  return options;
};
const name$6 = "@v-charts2/waterfall";
const version$6 = "1.0.1-alpha.4";
echartsLib.use([BarChart]);
const VeWaterfall = createChart$1({
  name: "VeWaterfall",
  chartHandler: waterfall
});
VeWaterfall.install = (app) => {
  logCopyRight(name$6, version$6);
  app.component(VeWaterfall.name, VeWaterfall);
};
function getTreeLegend(args) {
  const {
    dimension,
    rows
  } = args;
  const result = rows.map((row) => row[dimension]);
  return { data: result };
}
function getTreeTooltip(args) {
  const { tooltipFormatter } = args;
  return {
    trigger: "item",
    triggerOn: "mousemove",
    formatter: tooltipFormatter
  };
}
function getTreeSeries(args) {
  const {
    dimension,
    metrics,
    rows,
    seriesMap
  } = args;
  const series = [];
  rows.forEach((row) => {
    const label = row[dimension];
    const seriesItem = seriesMap[label];
    const result = {
      type: "tree",
      name: row[dimension],
      data: row[metrics]
    };
    if (seriesMap[row[dimension]]) {
      Object.keys(seriesItem).forEach((key) => {
        if (isObject(result[key])) {
          Object.assign(result[key], seriesItem[key]);
        } else {
          result[key] = seriesItem[key];
        }
      });
    }
    series.push(result);
  });
  return series;
}
const tree = (columns, rows, settings, extra) => {
  const {
    dimension = columns[0],
    metrics = columns[1],
    seriesMap = {}
  } = settings;
  const {
    legendVisible,
    tooltipFormatter,
    tooltipVisible
  } = extra;
  const series = getTreeSeries({
    dimension,
    metrics,
    rows,
    seriesMap
  });
  const legend = legendVisible && rows.length > 1 && getTreeLegend({
    dimension,
    rows
  });
  const tooltip = tooltipVisible && getTreeTooltip({ tooltipFormatter });
  return {
    series,
    legend,
    tooltip
  };
};
const name$5 = "@v-charts2/tree";
const version$5 = "1.0.1-alpha.4";
echartsLib.use([TreeChart]);
const VeTree = createChart$1({
  name: "VeTree",
  chartHandler: tree
});
VeTree.install = (app) => {
  logCopyRight(name$5, version$5);
  app.component(VeTree.name, VeTree);
};
function getScatterLegend(dataLabels, legendName) {
  return {
    data: dataLabels,
    formatter(name2) {
      return legendName[name2] != null ? legendName[name2] : name2;
    }
  };
}
function getScatterTooltip(args) {
  const { tooltipTrigger } = args;
  return {
    trigger: tooltipTrigger,
    formatter(item) {
      if (isArray(item)) {
        return item.map((i) => getTooltipContent(i, args)).join("");
      } else {
        return getTooltipContent(item, args);
      }
    }
  };
}
function getTooltipContent(item, args) {
  const {
    labelMap,
    columns,
    dataType,
    digit
  } = args;
  const tpl = [];
  const {
    color,
    seriesName,
    data: { value }
  } = item;
  tpl.push(`${itemPoint(color)} ${seriesName}<br>`);
  value.forEach((d, i) => {
    const name2 = labelMap[columns[i]] || columns[i];
    const num = isNaN(d) ? d : getFormated(d, dataType[columns[i]], digit);
    tpl.push(`${name2}: ${num}<br>`);
  });
  return tpl.join("");
}
function getScatterXAxis(args) {
  const {
    xAxisName,
    axisVisible,
    xAxisType,
    rows,
    dataLabels,
    dimension
  } = args;
  const data = [];
  dataLabels.forEach((dataLabel) => {
    const itemData = rows[dataLabel];
    itemData.forEach((item) => {
      const name2 = item[dimension];
      if (name2 && !~data.indexOf(name2))
        data.push(name2);
    });
  });
  return [{
    type: xAxisType,
    show: axisVisible,
    name: xAxisName,
    data
  }];
}
function getScatterYAxis(args) {
  const {
    min,
    max,
    scale,
    yAxisName,
    dataType,
    metrics,
    digit,
    axisVisible
  } = args;
  return {
    type: "value",
    show: axisVisible,
    scale,
    min,
    max,
    axisTick: { show: false },
    name: yAxisName,
    axisLabel: { formatter(val) {
      return getFormated(val, dataType[metrics[0]], digit);
    } }
  };
}
function getScatterSeries(args) {
  const {
    rows,
    dataLabels,
    columns,
    metrics,
    dimension,
    label,
    itemStyle,
    symbol,
    symbolSizeMax,
    symbolSize,
    symbolRotate,
    symbolOffset,
    cursor
  } = args;
  const extraMetrics = columns.filter((column) => {
    return !~metrics.indexOf(column) && column !== dimension;
  });
  const numbers = [];
  dataLabels.forEach((dataLabel) => {
    rows[dataLabel].forEach((row) => {
      numbers.push(row[metrics[1]]);
    });
  });
  const maxNum = Math.max.apply(null, numbers);
  const series = [];
  dataLabels.forEach((dataLabel) => {
    const result = [];
    const itemData = rows[dataLabel];
    itemData.forEach((item) => {
      const itemResult = { value: [] };
      itemResult.value.push(item[dimension], item[metrics[0]], item[metrics[1]]);
      extraMetrics.forEach((ext) => {
        itemResult.value.push(item[ext]);
      });
      itemResult.symbolSize = symbolSize || item[metrics[1]] / maxNum * symbolSizeMax;
      result.push(itemResult);
    });
    series.push({
      type: "scatter",
      data: result,
      name: dataLabel,
      label,
      itemStyle,
      symbol,
      symbolRotate,
      symbolOffset,
      cursor
    });
  });
  return series;
}
const scatter = (columns, rows, settings, extra) => {
  const {
    dimension = columns[0],
    metrics = [columns[1], columns[2]],
    dataType = {},
    xAxisType = "category",
    xAxisName,
    yAxisName,
    digit = 2,
    legendName = {},
    labelMap = {},
    tooltipTrigger = "item",
    axisVisible = true,
    symbolSizeMax = 50,
    symbol,
    symbolSize,
    symbolRotate,
    symbolOffset,
    cursor,
    min,
    max,
    scale,
    label,
    itemStyle
  } = settings;
  if (isArray(rows)) {
    const lineSettings = Object.assign({}, settings, {
      xAxisName: xAxisName ? [xAxisName] : void 0,
      yAxisName: yAxisName ? [yAxisName] : void 0,
      scale: scale ? [scale] : void 0,
      min: min ? [min] : void 0,
      max: max ? [max] : void 0,
      dimension: dimension ? [dimension] : void 0
    });
    const options = line(columns, rows, lineSettings, extra);
    if (!options || !options.series)
      return {};
    options.series.forEach((item) => {
      Object.assign(item, {
        type: "scatter",
        symbol,
        symbolSize: symbolSize || 10,
        symbolRotate,
        symbolOffset,
        cursor,
        label,
        itemStyle
      });
    });
    return options;
  }
  const {
    tooltipVisible,
    legendVisible
  } = extra;
  const dataLabels = Object.keys(rows);
  const legend = legendVisible && getScatterLegend(dataLabels, legendName);
  const tooltip = tooltipVisible && getScatterTooltip({
    tooltipTrigger,
    labelMap,
    columns,
    dataType,
    digit
  });
  const xAxis = getScatterXAxis({
    xAxisName,
    axisVisible,
    xAxisType,
    dataLabels,
    dimension,
    rows
  });
  const yAxis = getScatterYAxis({
    min,
    max,
    scale,
    yAxisName,
    dataType,
    metrics,
    digit,
    axisVisible
  });
  const series = getScatterSeries({
    rows,
    dataLabels,
    columns,
    metrics,
    dimension,
    label,
    itemStyle,
    symbol,
    symbolSizeMax,
    symbolSize,
    symbolRotate,
    symbolOffset,
    cursor
  });
  return {
    legend,
    tooltip,
    xAxis,
    yAxis,
    series
  };
};
const name$4 = "@v-charts2/scatter";
const version$4 = "1.0.1-alpha.4";
echartsLib.use([ScatterChart]);
const VeScatter = createChart$1({
  name: "VeScatter",
  chartHandler: scatter
});
VeScatter.install = (app) => {
  logCopyRight(name$4, version$4);
  app.component(VeScatter.name, VeScatter);
};
function getTooltip$2(args) {
  const {
    itemDataType,
    linksDataType,
    digit
  } = args;
  return {
    trigger: "item",
    formatter(item) {
      const tpl = [];
      const {
        name: name2,
        data,
        value,
        color
      } = item;
      tpl.push(itemPoint(color));
      tpl.push(`${name2} : `);
      if (data && data.source) {
        tpl.push(`${getFormated(value, linksDataType, digit)}<br />`);
      } else {
        tpl.push(`${getFormated(value, itemDataType, digit)}<br />`);
      }
      return tpl.join("");
    }
  };
}
function getSeries$2(args) {
  const {
    rows,
    dimension,
    metrics,
    links,
    valueFull,
    useDataValue,
    label,
    itemStyle,
    lineStyle
  } = args;
  const dataMap = {};
  const seriesData = rows.map((row) => {
    dataMap[row[dimension]] = row[metrics];
    return {
      name: row[dimension],
      value: row[metrics]
    };
  });
  let innerLinks = null;
  if (useDataValue) {
    innerLinks = links.map((link) => {
      return Object.assign({}, link, { value: dataMap[link.target] });
    });
  } else if (!valueFull) {
    innerLinks = links.map((link) => {
      return link.value == null ? Object.assign({}, link, { value: dataMap[link.target] }) : link;
    });
  } else {
    innerLinks = links;
  }
  const result = {
    type: "sankey",
    data: seriesData,
    links: innerLinks
  };
  if (label)
    result.label = label;
  if (itemStyle)
    result.itemStyle = itemStyle;
  if (lineStyle)
    result.lineStyle = lineStyle;
  return [result];
}
const sankey = (columns, rows, settings, extra) => {
  const {
    links,
    dimension = columns[0],
    metrics = columns[1],
    dataType = ["normal", "normal"],
    digit = 2,
    valueFull = false,
    useDataValue = false,
    label,
    itemStyle,
    lineStyle
  } = settings;
  if (!links) {
    console.warn("links is needed in settings!");
    return;
  }
  const itemDataType = dataType[0];
  const linksDataType = dataType[1];
  const tooltip = getTooltip$2({
    itemDataType,
    linksDataType,
    digit
  });
  const series = getSeries$2({
    rows,
    dimension,
    metrics,
    links,
    valueFull,
    useDataValue,
    label,
    itemStyle,
    lineStyle
  });
  return {
    tooltip,
    series
  };
};
const name$3 = "@v-charts2/sankey";
const version$3 = "1.0.1-alpha.4";
echartsLib.use([SankeyChart]);
const VeSankey = createChart$1({
  name: "VeSankey",
  chartHandler: sankey
});
VeSankey.install = (app) => {
  logCopyRight(name$3, version$3);
  app.component(VeSankey.name, VeSankey);
};
function getRadarLegend(rows, dimension, legendName) {
  const legendData = rows.map((row) => row[dimension]);
  return {
    data: legendData,
    formatter(name2) {
      return legendName[name2] != null ? legendName[name2] : name2;
    }
  };
}
function getRadarTooltip(dataType, radar2, digit) {
  const typeTemp = [];
  const nameTemp = [];
  radar2.indicator.map((item, index) => {
    typeTemp[index] = dataType[item.name];
    nameTemp[index] = item.name;
    return item;
  });
  return {
    formatter(item) {
      const tpl = [];
      tpl.push(itemPoint(item.color));
      tpl.push(`${item.name}<br />`);
      item.data.value.forEach((val, index) => {
        tpl.push(`${nameTemp[index]}: `);
        tpl.push(`${getFormated(val, typeTemp[index], digit)}<br />`);
      });
      return tpl.join("");
    }
  };
}
function getRadarSetting(rows, metrics, labelMap) {
  const settingBase = {
    center: ["50%", "60%"],
    indicator: [],
    shape: "circle",
    splitNumber: 5
  };
  const indicatorTemp = {};
  rows.forEach((items) => {
    metrics.forEach((item) => {
      const key = labelMap[item] != null ? labelMap[item] : item;
      if (!indicatorTemp[key]) {
        indicatorTemp[key] = [items[item]];
      } else {
        indicatorTemp[key].push(items[item]);
      }
    });
  });
  settingBase.indicator = Object.keys(indicatorTemp).map((key) => {
    return {
      name: key,
      max: Math.max.apply(null, indicatorTemp[key])
    };
  });
  return settingBase;
}
function getRadarSeries(args) {
  const {
    rows,
    dimension,
    metrics,
    radar: radar2,
    label,
    itemStyle,
    lineStyle,
    labelMap,
    areaStyle
  } = args;
  const radarIndexObj = {};
  radar2.indicator.forEach((item, index) => {
    const name2 = item.name;
    radarIndexObj[name2] = index;
  });
  const seriesData = rows.map((row) => {
    const serieData = {
      value: [],
      name: row[dimension]
    };
    Object.keys(row).forEach((key) => {
      if (~metrics.indexOf(key)) {
        const k = labelMap[key] != null ? radarIndexObj[labelMap[key]] : radarIndexObj[key];
        serieData.value[k] = row[key];
      }
    });
    return serieData;
  });
  const result = {
    name: dimension,
    type: "radar",
    data: seriesData
  };
  if (label)
    result.label = label;
  if (itemStyle)
    result.itemStyle = itemStyle;
  if (lineStyle)
    result.lineStyle = lineStyle;
  if (areaStyle)
    result.areaStyle = areaStyle;
  return [result];
}
const radar = (columns, rows, settings, extra) => {
  const {
    dataType = {},
    legendName = {},
    labelMap = {},
    dimension = columns[0],
    digit = 2,
    label,
    itemStyle,
    lineStyle,
    areaStyle
  } = settings;
  const {
    tooltipVisible,
    legendVisible
  } = extra;
  let metrics = columns.slice();
  if (settings.metrics) {
    metrics = settings.metrics;
  } else {
    metrics.splice(columns.indexOf(dimension), 1);
  }
  const legend = legendVisible && getRadarLegend(rows, dimension, legendName);
  const radar2 = getRadarSetting(rows, metrics, labelMap);
  const tooltip = tooltipVisible && getRadarTooltip(dataType, radar2, digit);
  const series = getRadarSeries({
    rows,
    dimension,
    metrics,
    radar: radar2,
    label,
    itemStyle,
    lineStyle,
    labelMap,
    areaStyle
  });
  const options = {
    legend,
    tooltip,
    radar: radar2,
    series
  };
  return options;
};
const name$2 = "@v-charts2/radar";
const version$2 = "1.0.1-alpha.4";
echartsLib.use([RadarChart]);
const VeRadar = createChart$1({
  name: "VeRadar",
  chartHandler: radar
});
VeRadar.install = (app) => {
  logCopyRight(name$2, version$2);
  app.component(VeRadar.name, VeRadar);
};
function getTooltip$1(dataType, digit, dataStore, metrics, color, labelMap) {
  return {
    formatter(item) {
      const tpl = [];
      if (!item.name)
        return "";
      tpl.push(`${item.name}<br>`);
      metrics.forEach((label, index) => {
        const title = labelMap[label] != null ? labelMap[label] : label;
        tpl.push(`${itemPoint(color[index])} ${title} : `);
        if (dataStore[item.name]) {
          tpl.push(getFormated(dataStore[item.name][label], dataType[label], digit));
        } else {
          tpl.push("-");
        }
        tpl.push("<br>");
      });
      return tpl.join(" ");
    }
  };
}
function getSeries$1(args) {
  const {
    position,
    selectData,
    dimension,
    metrics,
    rows,
    label,
    itemStyle,
    selectedMode,
    roam,
    center,
    aspectScale,
    boundingCoords,
    zoom,
    labelMap,
    scaleLimit,
    mapGrid
  } = args;
  const result = [];
  const mapBase = {
    type: "map",
    map: position
  };
  metrics.forEach((itemName) => {
    const itemResult = Object.assign({
      name: labelMap[itemName] != null ? labelMap[itemName] : itemName,
      data: [],
      selectedMode,
      roam,
      center,
      aspectScale,
      boundingCoords,
      zoom,
      scaleLimit
    }, mapBase);
    if (mapGrid) {
      Object.keys(mapGrid).forEach((key) => {
        itemResult[key] = mapGrid[key];
      });
    }
    setGeoLabel(itemStyle, itemResult, "itemStyle");
    setGeoLabel(label, itemResult, "label");
    rows.forEach((row) => {
      itemResult.data.push({
        name: row[dimension],
        value: row[itemName],
        selected: selectData
      });
    });
    result.push(itemResult);
  });
  return result;
}
function setGeoLabel(value, target, label) {
  if (typeof value === "object") {
    target[label] = value;
  } else if (value) {
    target[label] = { show: true };
  }
}
function getLegendMap(args) {
  const {
    metrics,
    legendName,
    labelMap
  } = args;
  if (!legendName && !labelMap)
    return { data: metrics };
  const data = labelMap ? metrics.map((item) => labelMap[item] == null ? item : labelMap[item]) : metrics;
  return {
    data,
    formatter(name2) {
      return legendName[name2] != null ? legendName[name2] : name2;
    }
  };
}
function registerMap(args, mapOrigin) {
  const {
    _once,
    registerSign,
    beforeRegisterMap,
    beforeRegisterMapOnce,
    registerSignOnce,
    position,
    specialAreas
  } = args;
  if (!_once[registerSign]) {
    if (beforeRegisterMap)
      mapOrigin = beforeRegisterMap(mapOrigin);
    if (beforeRegisterMapOnce && !_once[registerSignOnce]) {
      _once[registerSignOnce] = true;
      mapOrigin = beforeRegisterMapOnce(mapOrigin);
    }
    _once[registerSign] = true;
    echartsLib.registerMap(position, mapOrigin, specialAreas);
  }
}
const map = (columns, rows, settings, extra) => {
  const {
    position = "china",
    selectData = false,
    selectedMode,
    label = true,
    dataType = {},
    digit = 2,
    dimension = columns[0],
    roam,
    center,
    aspectScale,
    boundingCoords,
    zoom,
    scaleLimit,
    legendName = {},
    labelMap = {},
    mapGrid,
    itemStyle,
    positionJsonLink,
    beforeRegisterMap,
    beforeRegisterMapOnce,
    mapURLProfix = "https://unpkg.com/echarts@3.6.2/map/json/",
    specialAreas = {}
  } = settings;
  const mapOrigin = settings.mapOrigin;
  let metrics = columns.slice();
  if (settings.metrics) {
    metrics = settings.metrics;
  } else {
    metrics.splice(columns.indexOf(dimension), 1);
  }
  const {
    tooltipVisible,
    legendVisible,
    color,
    _once
  } = extra;
  const dataStore = {};
  rows.forEach((row) => {
    dataStore[row[dimension]] = row;
  });
  const tooltip = tooltipVisible && getTooltip$1(dataType, digit, dataStore, metrics, color, labelMap);
  const legend = legendVisible && getLegendMap({
    metrics,
    legendName,
    labelMap
  });
  const seriesParams = {
    position,
    selectData,
    label,
    itemStyle,
    dimension,
    metrics,
    rows,
    selectedMode,
    roam,
    center,
    aspectScale,
    boundingCoords,
    zoom,
    labelMap,
    scaleLimit,
    mapGrid
  };
  const series = getSeries$1(seriesParams);
  const registerOptions = {
    _once,
    beforeRegisterMap,
    beforeRegisterMapOnce,
    registerSign: `MAP_REGISTER_${position}`,
    registerSignOnce: `ONCE_MAP_REGISTER_${position}`,
    position,
    specialAreas
  };
  if (mapOrigin) {
    registerMap(registerOptions, mapOrigin);
    return {
      series,
      tooltip,
      legend
    };
  } else {
    return getMapJSON({
      position,
      positionJsonLink,
      beforeRegisterMapOnce,
      mapURLProfix
    }).then((json) => {
      registerMap(registerOptions, json);
      return {
        series,
        tooltip,
        legend
      };
    });
  }
};
const name$1 = "@v-charts2/map";
const version$1 = "1.0.1-alpha.4";
var createChart = {
  render(h2) {
    return h2("div", {
      class: [
        camelToKebab(this.$options.name || this.$options._componentTag)
      ],
      style: this.canvasStyle
    }, [
      h2("div", {
        style: this.canvasStyle,
        class: { "v-charts-mask-status": this.dataEmpty || this.loading },
        ref: "canvas"
      }),
      h2("div", {
        class: "v-charts-data-empty",
        style: { display: this.dataEmpty ? "" : "none" }
      }, "\u6682\u65E0\u6570\u636E"),
      h2("div", {
        style: { display: this.loading ? "" : "none" },
        class: "v-charts-component-loading"
      }, [
        h2("div", { class: "loader" }, [
          h2("div", { class: "loading-spinner" }, [
            h2("svg", {
              class: "circular",
              attrs: { viewBox: "25 25 50 50" }
            }, [
              h2("circle", {
                class: "path",
                attrs: {
                  cx: "50",
                  cy: "50",
                  r: "20",
                  fill: "none"
                }
              })
            ])
          ])
        ])
      ]),
      this.$slots.default
    ]);
  },
  props,
  created() {
    this.echarts = null;
    this.registeredEvents = [];
    this._once = {};
    this._store = {};
    this._watchers = [];
    this.initPayload = {};
  },
  data() {
    return {
      chartLib: null,
      echarts: null,
      registeredEvents: [],
      _once: {},
      _store: {},
      _watchers: [],
      initPayload: {}
    };
  },
  watch: {
    data: {
      deep: true,
      handler(v) {
        if (v) {
          changeHandler({
            props: this.$props,
            options: this.initPayload.options,
            echarts: this.echarts
          });
        }
      }
    },
    settings: {
      deep: true,
      handler(v) {
        if (v.type && this.chartLib)
          this.chartHandler = this.chartLib[v.type];
        this.initPayload.options.chartHandler = this.chartLib[v.type];
        changeHandler({
          props: this.$props,
          options: this.initPayload.options,
          echarts: this.echarts
        });
      }
    },
    width: "nextTickResize",
    height: "nextTickResize",
    events: {
      deep: true,
      handler() {
        createEventProxy({
          registeredEvents: this.registeredEvents,
          echarts: this.echarts,
          props: this.$props
        });
      }
    },
    theme: {
      deep: true,
      handler(val) {
        themeChange(this.initPayload, props);
      }
    },
    themeName: (val) => {
      themeChange(globalThis.initPayload, props);
    },
    resizeable: (val) => {
      globalThis.$nextTick(() => {
        resizeableHandler({
          resizeable: globalThis.resizeable,
          _once: globalThis._once,
          props: globalThis.$props,
          el: globalThis.$el,
          echarts: globalThis.echarts
        });
      });
    }
  },
  computed: {
    canvasStyle() {
      return {
        width: this.width,
        height: this.height,
        position: "relative"
      };
    },
    chartColor() {
      return this.colors || this.theme && this.theme.color || DEFAULT_COLORS;
    }
  },
  mounted() {
    const {
      echarts,
      $props: props2,
      $el: el,
      registeredEvents,
      chartHandler,
      _once,
      _store,
      $nextTick: nextTick2,
      _watchers
    } = this;
    const initPayload = {
      echarts,
      props: props2,
      canvas: this.$refs.canvas,
      options: {
        el,
        registeredEvents,
        chartHandler,
        chartColor: this.chartColor,
        _once,
        _store,
        readyCallback: (echarts2, chartOptions, echartsLib2) => {
          this.$emit("ready", echarts2, chartOptions, echartsLib2);
        },
        readyOnceCallback: (echarts2, chartOptions, echartsLib2) => {
          this.$emit("ready-once", echarts2, chartOptions, echartsLib2);
        }
      },
      nextTick: nextTick2
    };
    const { echarts: initEcharts } = init(initPayload);
    this.echarts = initEcharts;
    this.initPayload = initPayload;
    this.addWatchToProps({
      _watchers,
      props: props2,
      options: initPayload.options,
      echarts: this.echarts
    });
  },
  beforeDestroy() {
    clean({
      echarts: this.echarts,
      props: this.$props,
      _once: this._once
    });
  },
  methods: {
    addWatchToProps() {
      const watchedVariable = this._watchers.map((watcher) => watcher.expression);
      Object.keys(this.$props).forEach((prop) => {
        if (!~watchedVariable.indexOf(prop) && !~STATIC_PROPS.indexOf(prop)) {
          const opts = {};
          if (~["[object Object]", "[object Array]"].indexOf(getType(this.$props[prop]))) {
            opts.deep = true;
          }
          this.$watch(prop, () => {
            changeHandler({
              props: this.$props,
              options: this.initPayload.options,
              echarts: this.echarts
            });
          }, opts);
        }
      });
    },
    nextTickResize() {
      this.$nextTick(resize({
        props: this.$props,
        el: this.$el,
        echarts: this.echarts
      }));
    }
  },
  _numerify: numerify
};
echartsLib.use([MapChart]);
const VeMap = createChart({
  name: "VeMap",
  chartHandler: map
});
VeMap.install = (app) => {
  logCopyRight(name$1, version$1);
  app.component(VeMap.name, VeMap);
};
function getAxisList(rows, label) {
  const result = [];
  rows.forEach((row) => {
    if (!~result.indexOf(row[label]))
      result.push(row[label]);
  });
  return result;
}
function getData(args) {
  const {
    rows,
    innerXAxisList,
    innerYAxisList,
    xDim,
    yDim,
    metrics,
    type,
    extraMetrics
  } = args;
  let result = null;
  if (type === "cartesian") {
    result = rows.map((row) => {
      const xIndex = innerXAxisList.indexOf(row[xDim]);
      const yIndex = innerYAxisList.indexOf(row[yDim]);
      const value = metrics ? row[metrics] : 1;
      const extraData = extraMetrics.map((m) => row[m] || "-");
      return { value: [xIndex, yIndex, value].concat(extraData) };
    });
  } else {
    result = rows.map((row) => {
      const value = metrics ? row[metrics] : 1;
      return { value: [row[xDim], row[yDim], value] };
    });
  }
  return result;
}
function getAxis(list, name2) {
  return {
    type: "category",
    data: list,
    name: name2,
    nameLocation: "end",
    splitArea: { show: true }
  };
}
function getVisualMap(args) {
  const {
    innerMin: min,
    innerMax: max,
    type,
    heatColor,
    series
  } = args;
  const result = {
    min,
    max,
    calculable: true
  };
  let extra = null;
  if (type === "map") {
    extra = {
      orient: "vertical",
      left: 0,
      bottom: 0,
      inRange: { color: heatColor || HEAT_MAP_COLOR }
    };
    if (!series[0].data.length)
      extra.show = false;
  } else if (type === "bmap" || type === "amap") {
    extra = {
      show: false,
      orient: "vertical",
      left: 0,
      bottom: 0,
      inRange: { color: heatColor || HEAT_BMAP_COLOR }
    };
  } else {
    extra = {
      orient: "horizontal",
      left: "center",
      bottom: 10,
      dimension: 2,
      inRange: heatColor && { color: heatColor }
    };
  }
  return Object.assign(result, extra);
}
function getSeries(args) {
  const { chartData } = args;
  return [{
    type: "heatmap",
    data: chartData
  }];
}
function getTooltip(args) {
  const {
    dataType,
    innerXAxisList,
    innerYAxisList,
    digit,
    extraMetrics,
    metrics
  } = args;
  return {
    trigger: "item",
    formatter({
      color,
      data: { value: [xDim, yDim, value, ...extraData] }
    }) {
      const tpl = [];
      tpl.push(`${innerXAxisList[xDim]} ~ ${innerYAxisList[yDim]}<br>`);
      extraMetrics.forEach((m, index) => {
        tpl.push(`${m}: ${extraData[index]}<br>`);
      });
      tpl.push(`${itemPoint(color)} ${metrics}: ${getFormated(value, dataType, digit)}<br>`);
      return tpl.join("");
    }
  };
}
const heatmap = (columns, rows, settings, status) => {
  const {
    type = "cartesian",
    xAxisList,
    yAxisList,
    dimension = [columns[0], columns[1]],
    metrics = columns[2],
    dataType = "normal",
    min,
    max,
    digit,
    bmap,
    amap: amap2,
    geo,
    key,
    v = "2.0",
    position,
    positionJsonLink,
    beforeRegisterMap,
    pointSize = 10,
    blurSize = 5,
    heatColor,
    yAxisName,
    xAxisName,
    beforeRegisterMapOnce,
    mapURLProfix = "https://unpkg.com/echarts@3.6.2/map/json/",
    specialAreas = {}
  } = settings;
  const { tooltipVisible } = status;
  let innerXAxisList = xAxisList;
  let innerYAxisList = yAxisList;
  let chartData = [];
  const extraMetrics = [];
  const mainColumn = dimension.concat([metrics]);
  columns.forEach((column) => {
    if (!~mainColumn.indexOf(column))
      extraMetrics.push(column);
  });
  if (type === "cartesian") {
    if (!innerXAxisList || !innerXAxisList.length) {
      innerXAxisList = getAxisList(rows, dimension[0]);
    }
    if (!innerYAxisList || !innerYAxisList.length) {
      innerYAxisList = getAxisList(rows, dimension[1]);
    }
    chartData = getData({
      rows,
      innerXAxisList,
      innerYAxisList,
      xDim: dimension[0],
      yDim: dimension[1],
      metrics,
      type,
      extraMetrics
    });
  } else {
    chartData = getData({
      rows,
      xDim: dimension[0],
      yDim: dimension[1],
      metrics,
      type,
      extraMetrics
    });
  }
  let metricsList = metrics ? rows.map((row) => row[metrics]) : [0, 5];
  if (!metricsList.length)
    metricsList = [0];
  const innerMin = min || Math.min.apply(null, metricsList);
  const innerMax = max || Math.max.apply(null, metricsList);
  const xAxis = getAxis(innerXAxisList, xAxisName);
  const yAxis = getAxis(innerYAxisList, yAxisName);
  const series = getSeries({ chartData });
  const visualMap = getVisualMap({
    innerMin,
    innerMax,
    type,
    heatColor,
    series
  });
  const tooltip = tooltipVisible && getTooltip({
    dataType,
    innerXAxisList,
    innerYAxisList,
    digit,
    extraMetrics,
    metrics
  });
  const options = {
    visualMap,
    series
  };
  if (type === "bmap") {
    Object.assign(options.series[0], {
      coordinateSystem: "bmap",
      pointSize,
      blurSize
    });
    return getBmap(key, v).then((_) => {
      return Object.assign({ bmap }, options);
    });
  } else if (type === "map") {
    options.series[0].coordinateSystem = "geo";
    return getMapJSON({
      position,
      positionJsonLink,
      beforeRegisterMapOnce,
      mapURLProfix
    }).then((json) => {
      const geoAttr = Object.assign({ map: position }, geo);
      if (beforeRegisterMap)
        json = beforeRegisterMap(json);
      echartsLib.registerMap(position, json, specialAreas);
      return Object.assign({ geo: geoAttr }, options);
    });
  } else if (type === "amap") {
    Object.assign(options.series[0], {
      coordinateSystem: "amap",
      pointSize,
      blurSize
    });
    return getAmap(key, v).then((_) => {
      return Object.assign({ amap: amap2 }, options);
    });
  } else {
    return Object.assign({
      xAxis,
      yAxis,
      tooltip
    }, options);
  }
};
const name = "@v-charts2/heatmap";
const version = "1.0.1-alpha.4";
echartsLib.use([HeatmapChart, MapChart, VisualMapComponent]);
const VeHeatmap = createChart({
  name: "VeHeatmap",
  chartHandler: heatmap
});
VeHeatmap.install = (app) => {
  logCopyRight(name, version);
  app.component(VeHeatmap.name, VeHeatmap);
};
const charts = [
  VeLine,
  VeBar,
  VeHistogram,
  VePie,
  VeRing,
  VeCandle,
  VeFunnel,
  VeAmap,
  VeGauge,
  VeLiquidfill,
  VeWordcloud,
  VeWaterfall,
  VeTree,
  VeScatter,
  VeSankey,
  VeRadar,
  VeMap,
  VeHeatmap
];
function install(Vue) {
  logCopyRight(name$j, version$j);
  charts.forEach((component) => {
    Vue.component(component.name, component);
  });
}
var indexVue3 = {
  install,
  version: version$j,
  name: name$j,
  charts
};
export { indexVue3 as default };
